#!/usr/bin/env python3

import json
import logging
import os
import threading
import time
import uuid
from abc import abstractclassmethod
from dataclasses import dataclass
from types import TracebackType
from typing import Any, Callable, Dict, List, Optional, Type, Union

import httpx

from ..schemas.response import ReturnResponse
from .endpoints import (
    AuthEndpoint,
    BitableEndpoint,
    CalendarEndpoint,
    DocsEndpoint,
    ExtensionsEndpoint,
    MessageEndpoint,
)
from .typing import SyncAsync

logger = logging.getLogger(__name__)


@dataclass
class ClientOptions:
    """
    ClientOptions 类。

    用于 Client Options 相关能力的封装。
    """

    auth: Optional[str] = None
    timeout_ms: int = 60_000
    base_url: str = "https://open.feishu.cn/open-apis"
    retry_max_attempts: int = 3
    retry_backoff_seconds: float = 0.5
    token_refresh_buffer_seconds: int = 300
    token_cache_path: str = "/tmp/.feishu_token.json"
    token_file_cache_enabled: bool = True


@dataclass
class FeishuResponse:
    """
    FeishuResponse 兼容数据结构（保留导出，避免历史导入报错）。
    """

    code: int
    data: Dict[str, Any]
    chat_id: Optional[str] = None
    message_id: Optional[str] = None
    msg_type: Optional[str] = None
    sender: Optional[Dict[str, Any]] = None
    msg: Optional[str] = None
    expire: Optional[int] = None
    tenant_access_token: Optional[str] = None


class TokenProvider:
    """
    统一 token 缓存/刷新入口。
    """

    def __init__(
        self,
        fetcher: Callable[[], ReturnResponse],
        cache_path: str,
        refresh_buffer_seconds: int = 300,
        file_cache_enabled: bool = True,
    ) -> None:
        self._fetcher = fetcher
        self._cache_path = cache_path
        self._refresh_buffer_seconds = refresh_buffer_seconds
        self._file_cache_enabled = file_cache_enabled
        self._memory_token: Optional[str] = None
        self._memory_expires_at: int = 0
        self._lock = threading.RLock()

    def get_token(self) -> ReturnResponse:
        """
        获取可用 token（优先内存，再落盘，最后刷新）。
        """

        with self._lock:
            task_id = uuid.uuid4().hex[:8]
            start = time.monotonic()
            now = int(time.time())

            if self._is_valid(self._memory_token, self._memory_expires_at, now):
                self._log_task(task_id, "token.memory", "hit", start)
                return ReturnResponse.ok(
                    msg="token cache hit (memory)",
                    data={
                        "token": self._memory_token,
                        "expires_at": self._memory_expires_at,
                    },
                )

            if self._file_cache_enabled:
                cached = self._read_cache_file()
                token = cached.get("token")
                expires_at = int(cached.get("expires_at", 0) or 0)
                if self._is_valid(token, expires_at, now):
                    self._memory_token = token
                    self._memory_expires_at = expires_at
                    self._log_task(task_id, "token.file", "hit", start)
                    return ReturnResponse.ok(
                        msg="token cache hit (file)",
                        data={"token": token, "expires_at": expires_at},
                    )

            refresh_resp = self.refresh()
            self._log_task(
                task_id,
                "token.refresh",
                "ok" if refresh_resp.code == 0 else "fail",
                start,
            )
            return refresh_resp

    def refresh(self) -> ReturnResponse:
        """
        强制刷新 token。
        """

        with self._lock:
            resp = self._fetcher()
            if resp.code != 0:
                return resp

            payload = resp.data if isinstance(resp.data, dict) else {}
            token = payload.get("token")
            expires_at = int(payload.get("expires_at", 0) or 0)
            if not token or not expires_at:
                return ReturnResponse.fail(
                    code=4001,
                    msg="token response invalid",
                    data={"error": {"target": "token_provider.refresh"}},
                )

            self._memory_token = token
            self._memory_expires_at = expires_at
            if self._file_cache_enabled:
                self._write_cache_file(token=token, expires_at=expires_at)

            return ReturnResponse.ok(
                msg="token refreshed",
                data={"token": token, "expires_at": expires_at},
            )

    def peek_file_token(self) -> ReturnResponse:
        """
        仅读取文件缓存，不触发刷新。
        """

        with self._lock:
            cached = self._read_cache_file()
            token = cached.get("token")
            expires_at = int(cached.get("expires_at", 0) or 0)
            if token:
                return ReturnResponse.ok(
                    msg="token cache read",
                    data={"token": token, "expires_at": expires_at},
                )
            return ReturnResponse.no_data(msg="token cache empty")

    def _is_valid(self, token: Optional[str], expires_at: int, now: int) -> bool:
        if not token or not expires_at:
            return False
        return expires_at - now > self._refresh_buffer_seconds

    def _read_cache_file(self) -> Dict[str, Any]:
        if not self._file_cache_enabled:
            return {}
        if not os.path.exists(self._cache_path):
            return {}
        try:
            with open(self._cache_path, "r", encoding="utf-8") as f:
                payload = json.load(f)
            if isinstance(payload, dict):
                return payload
            return {}
        except (OSError, ValueError):
            return {}

    def _write_cache_file(self, token: str, expires_at: int) -> None:
        cache_dir = os.path.dirname(self._cache_path)
        if cache_dir:
            os.makedirs(cache_dir, exist_ok=True)

        payload = {"token": token, "expires_at": expires_at}
        temp_path = f"{self._cache_path}.tmp"
        with open(temp_path, "w", encoding="utf-8") as f:
            json.dump(payload, f, ensure_ascii=False)
        os.replace(temp_path, self._cache_path)

    def _log_task(self, task_id: str, target: str, result: str, start: float) -> None:
        duration_ms = int((time.monotonic() - start) * 1000)
        logger.info(
            "task_id=%s target=%s result=%s duration_ms=%s",
            task_id,
            target,
            result,
            duration_ms,
        )


class BaseClient:
    """
    BaseClient 类。

    用于 Base Client 相关能力的封装。
    """

    def __init__(
        self,
        app_id: str,
        app_secret: str,
        client: Union[httpx.Client, httpx.AsyncClient],
    ) -> None:
        self.app_id = app_id
        self.app_secret = app_secret
        self.options = ClientOptions()

        self._clients: List[Union[httpx.Client, httpx.AsyncClient]] = []
        self.client = client

        self.token_provider = TokenProvider(
            fetcher=self._fetch_token_from_api,
            cache_path=self.options.token_cache_path,
            refresh_buffer_seconds=self.options.token_refresh_buffer_seconds,
            file_cache_enabled=self.options.token_file_cache_enabled,
        )

        self.auth = AuthEndpoint(self)
        self.message = MessageEndpoint(self)
        self.bitable = BitableEndpoint(self)
        self.docs = DocsEndpoint(self)
        self.calendar = CalendarEndpoint(self)
        self.extensions = ExtensionsEndpoint(self)

    @property
    def client(self) -> Union[httpx.Client, httpx.AsyncClient]:
        """
        执行 client 相关逻辑。

        Returns:
            Any: 返回值。
        """

        return self._clients[-1]

    @client.setter
    def client(self, client: Union[httpx.Client, httpx.AsyncClient]) -> None:
        """
        执行 client 相关逻辑。
        """

        client.base_url = httpx.URL(f"{self.options.base_url}/")
        client.timeout = httpx.Timeout(timeout=self.options.timeout_ms / 1_000)
        client.headers = httpx.Headers({"User-Agent": "cc_feishu"})
        self._clients.append(client)

    def _build_request(
        self,
        method: str,
        path: str,
        query: Optional[Dict[str, Any]] = None,
        body: Optional[Dict[str, Any]] = None,
        data: Optional[Any] = None,
        files: Optional[Dict[str, Any]] = None,
        token: Optional[str] = None,
        headers: Optional[Dict[str, str]] = None,
    ) -> httpx.Request:
        request_headers = httpx.Headers(headers or {})
        if token:
            request_headers["Authorization"] = f"Bearer {token}"
        return self.client.build_request(
            method=method,
            url=path,
            params=query,
            json=body,
            headers=request_headers,
            files=files,
            data=data,
        )

    def _build_success_response(
        self, response: httpx.Response, response_json: Dict[str, Any]
    ) -> ReturnResponse:
        msg = str(response_json.get("msg") or response_json.get("message") or "OK")
        data = response_json.get("data")
        if data is None:
            extra = {
                key: value
                for key, value in response_json.items()
                if key not in {"code", "msg", "message"}
            }
            data = extra or None
        return ReturnResponse(code=0, msg=msg, data=data)

    def _build_error_response(
        self, response: Optional[httpx.Response], response_json: Dict[str, Any], msg: str
    ) -> ReturnResponse:
        http_status = response.status_code if response is not None else None
        err_code = response_json.get("code", http_status or 4001)
        try:
            code = int(err_code)
        except (TypeError, ValueError):
            code = 4001
        error = {
            "http_status": http_status,
            "errcode": response_json.get("code"),
            "errmsg": response_json.get("msg") or response_json.get("errmsg") or msg,
            "req_id": response_json.get("request_id") or response_json.get("req_id"),
        }
        payload = response_json.get("data")
        if not isinstance(payload, dict):
            payload = {"raw": payload}
        payload["error"] = error
        return ReturnResponse(code=code, msg=msg, data=payload)

    def _is_retryable_status(self, status_code: int) -> bool:
        return status_code == 429 or status_code >= 500

    def _should_retry_by_api_code(self, response_json: Dict[str, Any]) -> bool:
        if not response_json:
            return False
        code = response_json.get("code")
        if isinstance(code, int):
            return code in {429, 500, 502, 503, 504}
        return False

    def _is_invalid_token(self, response_json: Dict[str, Any], msg: str) -> bool:
        code = response_json.get("code")
        if code in {99991661, 99991663, 99991668}:
            return True
        lower_msg = msg.lower()
        return "invalid access token" in lower_msg or "tenant_access_token" in lower_msg

    def _sleep_backoff(self, attempt: int) -> None:
        time.sleep(self.options.retry_backoff_seconds * (2 ** (attempt - 1)))

    def _rewind_files(self, files: Optional[Dict[str, Any]]) -> None:
        if not files:
            return
        for value in files.values():
            file_obj: Optional[Any] = None
            if hasattr(value, "seek"):
                file_obj = value
            elif isinstance(value, tuple):
                for item in value:
                    if hasattr(item, "seek"):
                        file_obj = item
                        break
            if file_obj is not None:
                try:
                    file_obj.seek(0)
                except (OSError, ValueError):
                    continue

    @abstractclassmethod
    def request(
        self,
        path: str,
        method: str,
        query: Optional[Dict[Any, Any]] = None,
        body: Optional[Dict[Any, Any]] = None,
        auth: Optional[str] = None,
        data: Optional[Any] = None,
        files: Optional[Dict[Any, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        use_auth: bool = True,
    ) -> SyncAsync[ReturnResponse]:
        """
        发起请求。
        """

        pass


class Client(BaseClient):
    """
    Client 类。

    用于 Client 相关能力的封装。
    """

    client: httpx.Client

    def __init__(
        self,
        app_id: str,
        app_secret: str,
        client: Optional[httpx.Client] = None,
    ) -> None:
        if client is None:
            client = httpx.Client()
        super().__init__(app_id, app_secret, client)

    def __enter__(self) -> "Client":
        self.client = httpx.Client()
        self.client.__enter__()
        return self

    def __exit__(
        self,
        exc_type: Type[BaseException],
        exc_value: BaseException,
        traceback: TracebackType,
    ) -> None:
        self.client.__exit__(exc_type, exc_value, traceback)
        del self._clients[-1]

    def close(self) -> None:
        self.client.close()

    def _get_token(self) -> ReturnResponse:
        return self.token_provider.get_token()

    def _fetch_token_from_api(self) -> ReturnResponse:
        payload = {"app_id": self.app_id, "app_secret": self.app_secret}
        response = self.request(
            path="/auth/v3/tenant_access_token/internal",
            method="POST",
            body=payload,
            use_auth=False,
        )
        if response.code != 0:
            return response

        token_payload = response.data if isinstance(response.data, dict) else {}
        token = token_payload.get("tenant_access_token")
        expire_seconds = int(token_payload.get("expire", 0) or 0)
        if not token or expire_seconds <= 0:
            return ReturnResponse.fail(
                code=4001,
                msg="failed to parse tenant access token",
                data={"error": {"target": "auth.fetch_token"}},
            )
        expires_at = int(time.time()) + expire_seconds
        return ReturnResponse.ok(
            msg="token fetched",
            data={"token": token, "expires_at": expires_at},
        )

    def request(
        self,
        path: str,
        method: str,
        query: Optional[Dict[Any, Any]] = None,
        body: Optional[Dict[Any, Any]] = None,
        auth: Optional[str] = None,
        data: Optional[Any] = None,
        files: Optional[Dict[Any, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        use_auth: bool = True,
    ) -> ReturnResponse:
        task_id = uuid.uuid4().hex[:8]
        start = time.monotonic()
        request_target = f"{method.upper()} {path}"
        refreshed_once = False

        token: Optional[str] = None
        if use_auth:
            token_resp = self._get_token()
            if token_resp.code != 0:
                duration_ms = int((time.monotonic() - start) * 1000)
                logger.warning(
                    "task_id=%s target=%s result=token_fail duration_ms=%s",
                    task_id,
                    request_target,
                    duration_ms,
                )
                return token_resp
            token_data = token_resp.data if isinstance(token_resp.data, dict) else {}
            token = token_data.get("token")

        last_error: Optional[ReturnResponse] = None
        for attempt in range(1, self.options.retry_max_attempts + 1):
            self._rewind_files(files)
            request = self._build_request(
                method=method,
                path=path,
                query=query,
                body=body,
                data=data,
                files=files,
                token=auth or token,
                headers=headers,
            )

            try:
                response = self.client.send(request)
            except (httpx.TimeoutException, httpx.RequestError) as exc:
                retryable = attempt < self.options.retry_max_attempts
                if retryable:
                    self._sleep_backoff(attempt)
                    continue
                last_error = ReturnResponse.fail(
                    code=4001,
                    msg=str(exc.__class__.__name__),
                    data={"error": {"target": request_target}},
                )
                break

            response_json: Dict[str, Any]
            try:
                response_json = response.json()
                if not isinstance(response_json, dict):
                    response_json = {"data": response_json}
            except ValueError:
                response_json = {}

            msg = str(
                response_json.get("msg")
                or response_json.get("errmsg")
                or response.reason_phrase
                or "request failed"
            )

            if use_auth and self._is_invalid_token(response_json, msg) and not refreshed_once:
                refresh_resp = self.token_provider.refresh()
                refreshed_once = True
                if refresh_resp.code == 0:
                    refresh_data = (
                        refresh_resp.data if isinstance(refresh_resp.data, dict) else {}
                    )
                    token = refresh_data.get("token")
                    continue
                last_error = refresh_resp
                break

            api_code = response_json.get("code", 0 if response.status_code < 400 else response.status_code)
            try:
                api_code_int = int(api_code)
            except (TypeError, ValueError):
                api_code_int = 4001

            is_success = response.status_code < 400 and api_code_int == 0
            if is_success:
                result = self._build_success_response(response=response, response_json=response_json)
                duration_ms = int((time.monotonic() - start) * 1000)
                logger.info(
                    "task_id=%s target=%s result=ok duration_ms=%s",
                    task_id,
                    request_target,
                    duration_ms,
                )
                return result

            retryable = (
                self._is_retryable_status(response.status_code)
                or self._should_retry_by_api_code(response_json)
            )
            error_resp = self._build_error_response(
                response=response,
                response_json=response_json,
                msg=msg,
            )
            last_error = error_resp
            if retryable and attempt < self.options.retry_max_attempts:
                self._sleep_backoff(attempt)
                continue
            break

        duration_ms = int((time.monotonic() - start) * 1000)
        logger.warning(
            "task_id=%s target=%s result=fail duration_ms=%s",
            task_id,
            request_target,
            duration_ms,
        )
        if last_error is not None:
            return last_error
        return ReturnResponse.fail(
            code=4001,
            msg="unknown request failure",
            data={"error": {"target": request_target}},
        )



#!/usr/bin/env python3

import re
from typing import Literal, Optional, List

from requests.auth import HTTPBasicAuth
import requests

from .utils.response import ReturnResponse


class PyJira:
    """JIRA API客户端类
    
    提供JIRA操作的封装方法，支持任务创建、更新、查询等功能。
    建议使用OAuth 2.0认证以提高安全性。
    """
    
    def __init__(self, 
                 base_url: str=None,
                 proxies: dict=None,
                 token: str=None,
                 username: str=None,
                 password: str=None,
                 timeout: int=10
            ) -> None:
        '''
        _summary_

        Args:
            base_url (str, optional): _description_. Defaults to None.
            proxies (dict, optional): _description_. Defaults to None.
            token (str, optional): _description_. Defaults to None.
            username (str, optional): _description_. Defaults to None.
            password (str, optional): _description_. Defaults to None.
        '''
        self.base_url = base_url
        self.rest_version = 3
        self.proxies = None

        if 'atlassian.net' in self.base_url:
            self.deploy_type = 'cloud'
            self.rest_version = 3
        else:
            self.deploy_type = 'datacenter'
            self.rest_version = 2

        if self.deploy_type == 'cloud':
            self.headers = {
                "Accept": "application/json",
                "Content-Type": "application/json",
                "Authorization": f"Basic {token}"
            }
        else:
            self.headers = {
                "Accept": "application/json",
                "Content-Type": "application/json",
                "Authorization": f"Bearer {token}"
            }
            self.auth = HTTPBasicAuth(username, password)
        
        self.timeout = timeout
        # 使用 requests.Session 统一管理连接与 headers（不改变现有硬编码与参数传递逻辑）
        self.session = requests.Session()
        self.session.headers.update(self.headers)
        
    def get_project_boards(self, project_key: Literal['TEST'] = 'TEST') -> list:
        """获取指定项目的看板列表
        
        Args:
            project_key: 项目key，例如 'OPS'
            
        Returns:
            list: 看板列表
        """
        boards = self.jira.boards(projectKeyOrID=project_key)
        for board in boards:
            print(board.name)
        return boards

    def issue_get(self, issue_id_or_key: str, account_id: bool = False) -> ReturnResponse:
        """获取指定JIRA任务
        
        Args:
            issue_key: 任务key，例如 'TEST-50'
            account_id: 是否返回账户ID，默认为False
            
        Returns:
            dict: 任务信息或账户ID
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_id_or_key}"
        print(url)
        r = self.session.get(url, headers=self.headers, timeout=self.timeout)
        
        print(r.text)
        
        # 移除所有 key 以 customfield_ 开头且后面跟数字的字段
        fields = r.json()['fields']
        keys_to_remove = [k for k in fields if k.startswith('customfield_') and k[12:].isdigit()]
        for k in keys_to_remove:
            fields.pop(k)
        
        if r.status_code == 200:
            return ReturnResponse(code=0, msg=f'获取 issue [{issue_id_or_key}] 成功', data=fields)
        else:
            return ReturnResponse(code=1, msg=f'获取 issue [{issue_id_or_key}] 失败')

    def issue_get_by_key(self, issue_id_or_key:str='', get_key: Literal['assignee', 'summary', 'description', 'status']=None):
        '''
        _summary_

        Args:
            issue_id_or_key (str, optional): _description_. Defaults to ''.
            get_key: status 就是 transitions

        Raises:
            ValueError: _description_

        Returns:
            _type_: _description_
        '''
        r = self.issue_get(issue_id_or_key=issue_id_or_key)
        if r.code == 0:
            if get_key:
                # print(get_key)
                # print(r.data['assignee'])
                try:
                    return r.data[get_key]
                except KeyError as e:
                    return r.data
        else:
            raise ValueError(f"{issue_id_or_key} 的 {get_key} 未查找到")
            

    def issue_create(self, 
                    project_id: int = 10000, 
                    summary: Optional[str] = None, 
                    description: str = None, 
                    description_adf: bool=True,
                    issue_type: Literal['任务'] = '任务', 
                    priority: Literal['Highest', 'High', 'Medium', 'Low', 'Lowest'] = 'Medium',
                    reporter: dict=None,
                    assignee: dict={},
                    parent_key: Optional[str] = None) -> ReturnResponse:
        """创建一个新的JIRA任务
        
        Args:
            project_id: 项目ID，默认为 10000
            summary: 任务标题
            description: 任务描述，默认为空字符串，支持普通文本，会自动转换为ADF格式
            issue_type: 任务类型，默认为 'Task'
            priority: 优先级，默认为 'Medium'
            parent_key: 父任务key，如果提供则创建子任务，默认为None
            reporter(dict): { "name": reporter}, : {"id" : xx}
            assignee: {"accountId": "xxx"}
            
        Returns:
            Response: 创建结果，包含任务信息或错误信息
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue"
        data = {
            "fields": {
                'project': { "id": project_id},
                'summary': summary,
                'issuetype': { "name": issue_type},
                'priority': { "name": priority},
                # 'reporter': reporter
            }
        }
        if assignee:
            data['fields']['assignee'] = assignee
        if description:
            # 将普通文本转换为ADF格式
            if description_adf:
                data['fields']['description'] = self.text_to_adf(description)
            else:
                data['fields']['description'] = description
            
        if parent_key:
            data['fields']['parent'] = {"key": parent_key}
    
        if reporter:
            if isinstance(reporter, str):
                data['fields']['reporter'] = {"name": reporter}
            else:
                data['fields']['reporter'] = reporter

        r = self.session.post(
            url,
            headers=self.headers,
            json=data,
            timeout=self.timeout,
            proxies=self.proxies
        )
        if r.status_code == 201:
            return ReturnResponse(code=0, msg=f'创建 issue [{summary}] 成功', data=r.json())
        else:
            return ReturnResponse(code=1, msg=f'创建 issue [{summary}] 失败, {r.text}')
        

    def issue_update(self, 
                     issue_key: str, 
                     summary: Optional[str] = None, 
                     description: str = None, 
                     issue_type: str = None,
                     priority: Literal['Blocker', 'Critical', 'Major', 'Minor'] = None,
                     labels: Optional[list[str]] = None,
                     parent_key: str=None) -> dict:
        """更新JIRA任务
        
        Args:
            issue_key: 任务key，例如 'TEST-50'
            summary: 任务标题，默认为None
            description: 任务描述，默认为空字符串，支持普通文本，会自动转换为ADF格式
            issue_type: 任务类型，默认为 'Task'
            priority: 优先级，默认为 'Minor'
            assignee: 分配人，默认为 'MingMing Hou'
            status: 任务状态，默认为None
            labels: 标签列表，默认为None
            
        Returns:
            dict: 更新后的任务信息或错误信息
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_key}"
        data = { "fields": {} }
        if summary:
            data['fields']['summary'] = summary
        if description is not None:
            description = self.format_description(description)
            # 将普通文本转换为ADF格式
            data['fields']['description'] = self.text_to_adf(description)
        if issue_type:
            data['fields']['issuetype'] = issue_type
        if priority:
            data['fields']['priority'] = {"name": priority}
        if labels:
            data['fields']['labels'] = labels
            
        if parent_key:
            data['fields']['parent'] = {"key": parent_key}
        
        # print(data)
        r = self.session.put(url, headers=self.headers, json=data, timeout=self.timeout)
        if r.status_code == 204:
            return ReturnResponse(code=0, msg=f'更新 issue [{issue_key}] 成功', data=r.text)
        else:
            try:
                error_data = r.json()
                error_message = f'更新 issue [{issue_key}] 失败, status: {r.status_code}, 错误: {error_data}'
            except:
                error_message = f'更新 issue [{issue_key}] 失败, status: {r.status_code}, 响应: {r.text}'
            return ReturnResponse(code=1, msg=error_message, data=r.text)

    def issue_assign(self, issue_id_or_key: str='', name: str=None, display_name: str=None, account_id: str=None) -> ReturnResponse:
        '''
        _summary_

        Args:
            issue_id_or_key (str, optional): _description_. Defaults to ''.
            name (str, optional): _description_. Defaults to None.
            display_name (str, optional): _description_. Defaults to None.
            account_id (str, optional): _description_. 侯明明: 612dbc1d1dbcd90069240013 包柿林: 712020:66dfa4a9-383e-4aa2-abdd-521adfccf967

        Returns:
            ReturnResponse: _description_
        '''    
        update = False
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_id_or_key}/assignee"
        data = {}
        if name:
            data["name"] = name
            assignee_message = name
            update = True
            
        if display_name:
            data["displayName"] = display_name
            assignee_message = display_name
            update = True
            
        if account_id:
            account_info = self.issue_get_by_key(issue_id_or_key=issue_id_or_key, get_key='assignee')
            if account_info is None:
                update = True
            else:
                current_account_id = account_info['accountId']
                if current_account_id != account_id:
                    update = True
            
            data["accountId"] = account_id
            assignee_message = account_id

        if update:
            r = self.session.put(url, headers=self.headers, json=data, timeout=self.timeout)
            if r.status_code == 204:
                return ReturnResponse(code=0, msg=f'更新 issue [{issue_id_or_key}] 的分配人成功, 经办人被更新为 {assignee_message}', data=r.text)
            return ReturnResponse(code=1, msg=f'更新 issue [{issue_id_or_key}] 的分配人失败, status code: {r.status_code}, 报错: {r.text}')
        return ReturnResponse(code=0, msg=f'issue [{issue_id_or_key}] 的分配人已经为 {assignee_message}, 不需要更新')

    def issue_comment_add(self, issue_key: str, comment: str) -> dict:
        """添加JIRA任务评论
        
        Args:
            issue_key: 任务key，例如 'TEST-50'
            comment: 评论内容
            
        Returns:
            dict: 评论信息
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_key}/comment/"
        comment_adf = self.text_to_adf(text=self.format_description(comment))
        r = self.session.post(url, headers=self.headers, json={"body": comment_adf}, timeout=self.timeout)
        if r.status_code == 201:
            return ReturnResponse(code=0, msg=f'添加评论 [{comment}] 成功', data=r.json())
        else:
            return ReturnResponse(code=1, msg=f'添加评论 [{comment}] 失败, 返回值: {r.text}', data=r.json())
    
    def issue_search(self, jql: str, max_results: int = 1000, fields: Optional[List[str]] = None) -> ReturnResponse:
        """使用JQL搜索JIRA任务（支持自动分页获取所有结果）
        https://developer.atlassian.com/cloud/jira/platform/rest/v3/api-group-issue-search/#api-rest-api-3-search-jql-get
        
        Args:
            jql: JQL查询字符串
            max_results: 最大返回结果数，默认1000。会自动分页获取
            fields: 需要返回的字段列表，默认返回所有字段
            
        Returns:
            ReturnResponse: 包含任务列表的响应数据
        """
        url = f"{self.base_url}/rest/api/3/search/jql"
        
        # Jira API 单次请求最多返回100条，需要分页
        page_size = 100
        all_issues = []
        seen_keys = set()  # 用于去重
        next_page_token = None
        page_count = 0
        
        while len(all_issues) < max_results:
            page_count += 1
            
            # 构建查询参数
            params = {
                "jql": jql,
                "maxResults": page_size
            }
            
            # 如果有下一页的 token，添加到参数中（按文档使用 nextPageToken 参数名）
            if next_page_token:
                params["nextPageToken"] = next_page_token
            
            # 如果指定了字段，添加到查询参数中
            if isinstance(fields, list):
                params["fields"] = ",".join(fields)
            else:
                params["fields"] = fields
            
            r = self.session.get(url, headers=self.headers, params=params, timeout=self.timeout)
            
            if r.status_code != 200:
                return ReturnResponse(code=1, msg=f'获取 issue 失败, status code: {r.status_code}, 报错: {r.text}')
            
            data = r.json()
            issues = data.get('issues', [])
            
            if not issues:
                break
            
            # 去重：只添加未见过的 issue
            new_issues_count = 0
            for issue in issues:
                issue_key = issue.get('key')
                if issue_key and issue_key not in seen_keys:
                    all_issues.append(issue)
                    seen_keys.add(issue_key)
                    new_issues_count += 1
                
            # 检查是否是最后一页
            is_last = data.get('isLast', True)
            next_page_token = data.get('nextPageToken')
            
            # 如果是最后一页或没有新数据，退出循环
            if is_last or new_issues_count == 0:
                break
        
        # 如果获取的数据超过 max_results，截断到指定数量
        if len(all_issues) > max_results:
            all_issues = all_issues[:max_results]
        
        # 返回合并后的结果
        result_data = {
            'issues': all_issues,
            'total': len(all_issues),
            'startAt': 0
        }
        
        return ReturnResponse(code=0, msg=f'成功获取 {len(all_issues)} 个唯一 issue（共请求 {page_count} 页）', data=result_data)


    def get_boards(self) -> ReturnResponse:
        """获取所有看板信息并打印
        
        使用REST API获取看板信息，主要用于调试和查看。
        """
        url = f"{self.base_url}/rest/agile/1.0/board"
        response = self.session.get(url, headers=self.headers, timeout=self.timeout)
        response.raise_for_status()
        return ReturnResponse(code=0, msg='', data=response.json())
 
    def get_issue_fields(self) -> ReturnResponse:
        """
        获取issue fields。

        Returns:
            Any: 返回值。
        """
        url = f"{self.base_url}/rest/api/3/field"
        response = self.session.get(url, headers=self.headers, timeout=self.timeout)
        response.raise_for_status()
        return ReturnResponse(code=0, msg='', data=response.json())
 
    def get_project(self, project_id_or_key: Literal['TEST']='TEST') -> ReturnResponse:
        """
        获取project。

        Args:
            project_id_or_key: project_id_or_key 参数。

        Returns:
            Any: 返回值。
        """
        if self.deploy_type == 'cloud':
            url = f"{self.base_url}/rest/api/3/project/{project_id_or_key}"
        else:
            url = f"{self.base_url}/rest/api/2/project"
            
        response = self.session.get(url, headers=self.headers, timeout=self.timeout)
        if response.status_code == 401:
            return ReturnResponse(code=1, msg=response.text)
        else:
            return ReturnResponse(code=0, msg='', data=response.json())

    # def get_issue(self, issue_id_or_key: str) -> ReturnResponse:
    #     url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_id_or_key}"
    #     response = self.session.get(url, headers=self.headers, timeout=self.timeout)
    #     if response.status_code == 200:
    #         return ReturnResponse(code=0, msg=f'根据输入的 {issue_id_or_key} 成功获取到 issue', data=response.json())
    #     else:
    #         return ReturnResponse(code=1, msg=response.text)
 
    def get_metadata_for_project_issue_types(self, project_id_or_key: Literal['TEST', 'MO', '10000']='TEST'):
        """
        获取metadata for project issue types。

        Args:
            project_id_or_key: project_id_or_key 参数。

        Returns:
            Any: 返回值。
        """
        url = f"{self.base_url}/rest/api/2/issue/createmeta/{project_id_or_key}/issuetypes"
        response = self.session.get(url, headers=self.headers, timeout=self.timeout)
        if response.status_code == 200:
            return ReturnResponse(code=0, msg='', data=response.json())
        else:
            return ReturnResponse(code=1, msg=f"{response.status_code}: {response.text}")
 
    def get_metadata_for_issue_type_used_for_create_issue(self, project_id_or_key: Literal['TEST', 'MO']='TEST', issue_type_id: Literal['10000']='10000'):
        """
        获取metadata for issue type used for create issue。

        Args:
            project_id_or_key: project_id_or_key 参数。
            issue_type_id: 资源 ID。

        Returns:
            Any: 返回值。
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue/createmeta/{project_id_or_key}/issuetypes/{issue_type_id}"
        r = self.session.get(url, headers=self.headers, timeout=self.timeout)
        return r
    
    def get_user(self, username: str=None, key: str=None):
        """
        获取user。

        Args:
            username: username 参数。
            key: key 参数。

        Returns:
            Any: 返回值。
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/user"
        r = self.session.get(url, headers=self.headers, params={'username': username}, timeout=self.timeout)
        return r
    
    def get_issue_property(self, issue_id_or_key):
        """
        获取issue property。

        Args:
            issue_id_or_key: issue_id_or_key 参数。

        Returns:
            Any: 返回值。
        """
        url = f"{self.base_url}/rest/api/2/issue/{issue_id_or_key}/properties"
        r = self.session.get(url, headers=self.headers, timeout=self.timeout)
        return r
    
    def find_user(self, query: str) -> ReturnResponse:
        '''
        查找用户

        Args:
            query (str): 建议输入用户的邮箱

        Returns:
            _type_: 
        '''
        url = f"{self.base_url}/rest/api/{self.rest_version}/user/search"
        r = self.session.get(url, headers=self.headers, params={'query': query}, timeout=self.timeout)
        if r.status_code == 200:
            return ReturnResponse(code=0, msg=f'查找到 {len(r.json())} 个用户', data=r.json()[0])
        else:
            return ReturnResponse(code=1, msg=f'获取用户失败, status code: {r.status_code}, 报错: {r.text}')
    
    def get_issue_transitions(self, issue_id_or_key) -> ReturnResponse:
        """
        获取issue transitions。

        Args:
            issue_id_or_key: issue_id_or_key 参数。

        Returns:
            Any: 返回值。
        """
        url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_id_or_key}/transitions"
        r = self.session.get(url, headers=self.headers, timeout=self.timeout)
        if r.status_code == 200:
            return ReturnResponse(code=0, msg='', data=r.json())
        else:
            return ReturnResponse(code=1, msg=f'获取 issue [{issue_id_or_key}] 的 transitions 失败, status code: {r.status_code}, 报错: {r.text}')
  
    def get_issue_transitions_by_name(self, issue_id_or_key, name) -> ReturnResponse:
        """
        获取issue transitions by name。

        Args:
            issue_id_or_key: issue_id_or_key 参数。
            name: name 参数。

        Returns:
            Any: 返回值。
        """
        r = self.get_issue_transitions(issue_id_or_key=issue_id_or_key)
        for transition in r.data['transitions']:
            if transition['name'] == name:
                return ReturnResponse(code=0, msg=f'获取 issue [{issue_id_or_key}] 的 transitions 成功, 返回值: {transition["id"]}', data=transition['id'])
        raise ValueError(f'获取 issue [{issue_id_or_key}] 的 transitions 失败, 没有找到状态为 {name} 的 transition')

    def issue_transition(self, issue_id_or_key, transition_name) -> ReturnResponse:
        """
        执行 issue transition 相关逻辑。

        Args:
            issue_id_or_key: issue_id_or_key 参数。
            transition_name: transition_name 参数。

        Returns:
            Any: 返回值。
        """
        update = False
        status = self.issue_get_by_key(issue_id_or_key=issue_id_or_key, get_key='status')
        if status:
            if transition_name != status['name']:
                update = True
        if update:
            url = f"{self.base_url}/rest/api/{self.rest_version}/issue/{issue_id_or_key}/transitions"
            data = {
                "transition": {
                    "id": self.get_issue_transitions_by_name(issue_id_or_key=issue_id_or_key, name=transition_name).data
                }
            }
            r = self.session.post(url, headers=self.headers, json=data, timeout=self.timeout)
            if r.status_code == 204:
                return ReturnResponse(code=0, msg=f'更新 issue [{issue_id_or_key}] 的状态成功, 状态被更新为 {transition_name}', data=r.text)
            else:
                return ReturnResponse(code=1, msg=f'更新 issue [{issue_id_or_key}] 的状态失败, status code: {r.status_code}, 报错: {r.text}')
        else:
            return ReturnResponse(code=0, msg=f'issue [{issue_id_or_key}] 的状态已经为 {status["name"]}, 不需要更新')

    def format_description(self, description):
        '''
        格式化描述
        '''
        if not description:
            return ""
        
        new_description = description
        
        # 内容清理正则表达式
        CONTENT_CLEANUP_PATTERNS = [
            r'\*\*工作历时\*\*: \d+小时',
            r'\*\*工作历时\*\*:.*小时',
            r'!\[image\]\(.*?\)',
            r'!\[file\]\(.*?\)',
            r'\[Jira Link\]\(.*?\)',
            r'bear://x-callback-url/open-note\?id=.*'
        ] 

        for pattern in CONTENT_CLEANUP_PATTERNS:
            new_description = re.sub(pattern, '', new_description)
        return new_description.strip()

    def text_to_adf(self, text: str) -> dict:
        """
        将普通文本转换为Atlassian Document Format (ADF)格式。
        
        Args:
            text: 要转换的文本内容
            
        Returns:
            dict: ADF格式的文档对象
        """
        if not text:
            return {
                "type": "doc",
                "version": 1,
                "content": [
                    {
                        "type": "paragraph",
                        "content": [
                            {
                                "type": "text",
                                "text": ""
                            }
                        ]
                    }
                ]
            }
        
        # 将文本按行分割
        lines = text.split('\n')
        content = []
        
        for line in lines:
            if line.strip():  # 非空行
                paragraph_content = [
                    {
                        "type": "text",
                        "text": line
                    }
                ]
                content.append({
                    "type": "paragraph",
                    "content": paragraph_content
                })
            else:  # 空行，添加换行符
                if content:  # 如果前面有内容，添加换行
                    content.append({
                        "type": "paragraph",
                        "content": [
                            {
                                "type": "text",
                                "text": ""
                            }
                        ]
                    })
        
        # 如果没有内容，至少添加一个空段落
        if not content:
            content = [
                {
                    "type": "paragraph",
                    "content": [
                        {
                            "type": "text",
                            "text": ""
                        }
                    ]
                }
            ]
        
        return {
            "type": "doc",
            "version": 1,
            "content": content
        }

 
if __name__ == '__main__':
    pass
    



"""Aliyun ECS resource operations."""

from __future__ import annotations

import json
from typing import Any

from alibabacloud_ecs20140526 import models as ecs_models

from ...schemas.response import ReturnResponse


class ECSResource:
    """Aliyun ECS read-only resource wrapper."""

    def __init__(self, client: Any) -> None:
        """Initialize ECS resource.

        Args:
            client: AliyunClient instance.
        """
        self._c = client

    @staticmethod
    def _extract_instances(body_map: dict[str, Any]) -> list[dict[str, Any]]:
        """Extract ECS instances from SDK map.

        Args:
            body_map: SDK response body map.

        Returns:
            list[dict[str, Any]]: Instance list.
        """
        instances = body_map.get("Instances", {}).get("Instance", [])
        if isinstance(instances, list):
            return instances
        return []

    @staticmethod
    def _extract_eip_addresses(body_map: dict[str, Any]) -> list[dict[str, Any]]:
        """Extract EIP addresses from SDK map.

        Args:
            body_map: SDK response body map.

        Returns:
            list[dict[str, Any]]: EIP address list.
        """
        eips = body_map.get("EipAddresses", {}).get("EipAddress", [])
        if isinstance(eips, list):
            return eips
        return []

    @staticmethod
    def _extract_disks(body_map: dict[str, Any]) -> list[dict[str, Any]]:
        """Extract disk list from SDK map.

        Args:
            body_map: SDK response body map.

        Returns:
            list[dict[str, Any]]: Disk list.
        """
        disks = body_map.get("Disks", {}).get("Disk", [])
        if isinstance(disks, list):
            return disks
        return []

    @staticmethod
    def _extract_collection(
        body_map: dict[str, Any],
        *,
        container_key: str,
        item_key: str,
    ) -> list[dict[str, Any]]:
        """Extract list items from a nested SDK response body.

        Args:
            body_map: SDK response body map.
            container_key: Top-level collection key.
            item_key: Item list key inside the collection.

        Returns:
            list[dict[str, Any]]: Extracted items.
        """
        container = body_map.get(container_key) or {}
        if not isinstance(container, dict):
            return []
        items = container.get(item_key) or []
        if isinstance(items, list):
            return [item for item in items if isinstance(item, dict)]
        return []

    def _list_paginated(
        self,
        *,
        action: str,
        request_cls: Any,
        sdk_method_name: str,
        container_key: str,
        item_key: str,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """Run a paginated ECS list request and flatten all pages.

        Args:
            action: Action label for logging and exception mapping.
            request_cls: SDK request model class.
            sdk_method_name: ECS SDK method name.
            container_key: Top-level collection key in response body.
            item_key: Item list key inside the collection.
            region: Optional region override.
            page_size: Number of records per page.
            **kwargs: Additional request parameters.

        Returns:
            ReturnResponse: ``data`` is a list of raw resource dicts.
        """
        region_id = region or self._c.cfg.region
        page_number = 1
        items_all: list[dict[str, Any]] = []

        while True:
            req = request_cls(
                region_id=region_id,
                page_size=page_size,
                page_number=page_number,
                **kwargs,
            )
            resp = self._c.call(
                action,
                lambda: getattr(self._c.ecs, sdk_method_name)(req),
            )
            body = getattr(resp, "body", None)
            body_map = body.to_map() if hasattr(body, "to_map") else {}

            items = self._extract_collection(
                body_map,
                container_key=container_key,
                item_key=item_key,
            )
            if not items:
                break
            items_all.extend(items)

            total_count_raw = body_map.get("TotalCount")
            if isinstance(total_count_raw, str) and total_count_raw.isdigit():
                total_count = int(total_count_raw)
            elif isinstance(total_count_raw, int):
                total_count = total_count_raw
            else:
                total_count = None

            if total_count is not None:
                if page_number * page_size >= total_count:
                    break
            elif len(items) < page_size:
                break

            page_number += 1

        return ReturnResponse(code=0, msg="success", data=items_all)

    def list(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS instances.

        Args:
            region: Optional region override.
            page_size: Number of instances per page.
            **kwargs: Pass-through params for ``DescribeInstancesRequest``.

        Returns:
            ReturnResponse: ``data`` is list of instance dicts.
        """
        region_id = region or self._c.cfg.region
        page_number = 1
        instances_all: list[dict[str, Any]] = []

        while True:
            req = ecs_models.DescribeInstancesRequest(
                region_id=region_id,
                page_size=page_size,
                page_number=page_number,
                **kwargs,
            )
            resp = self._c.call("ecs_list", lambda: self._c.ecs.describe_instances(req))
            body = getattr(resp, "body", None)
            body_map = body.to_map() if hasattr(body, "to_map") else {}

            instances = self._extract_instances(body_map)
            if not instances:
                break
            instances_all.extend(instances)

            total_count = int(body_map.get("TotalCount", 0) or 0)
            if page_number * page_size >= total_count:
                break
            page_number += 1
        return ReturnResponse(code=0, msg="success", data=instances_all)

    def get_instance(
        self,
        instance_id: str,
        *,
        region: str | None = None,
        **kwargs: Any,
    ) -> ReturnResponse:
        """Get a single ECS instance by id.

        Args:
            instance_id: ECS instance id.
            region: Optional region override.
            **kwargs: Additional request parameters.

        Returns:
            ReturnResponse: ``data`` is instance dict or ``None`` when missing.
        """
        region_id = region or self._c.cfg.region
        request_kwargs = dict(kwargs)
        request_kwargs.setdefault("instance_ids", json.dumps([instance_id]))
        req = ecs_models.DescribeInstancesRequest(
            region_id=region_id,
            page_size=1,
            page_number=1,
            **request_kwargs,
        )
        resp = self._c.call("ecs_get_instance", lambda: self._c.ecs.describe_instances(req))
        body = getattr(resp, "body", None)
        body_map = body.to_map() if hasattr(body, "to_map") else {}
        instances = self._extract_instances(body_map)
        return ReturnResponse(code=0, msg="success", data=instances[0] if instances else None)

    def list_instance_ids(self, *, region: str | None = None, **kwargs: Any) -> ReturnResponse:
        """List ECS instance ids.

        Args:
            region: Optional region override.
            **kwargs: Additional list filters.

        Returns:
            ReturnResponse: ``data`` is list of instance ids.
        """
        response = self.list(region=region, **kwargs)
        instances = response.data if isinstance(response.data, list) else []
        instance_ids = [str(item.get("InstanceId")) for item in instances if item.get("InstanceId")]
        return ReturnResponse(code=response.code, msg=response.msg, data=instance_ids)

    def list_disks(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS disks.

        Args:
            region: Optional region override.
            page_size: Number of disks per page.
            **kwargs: Pass-through params for ``DescribeDisksRequest``.

        Returns:
            ReturnResponse: ``data`` is list of disk dicts.
        """
        return self._list_paginated(
            action="ecs_list_disks",
            request_cls=ecs_models.DescribeDisksRequest,
            sdk_method_name="describe_disks",
            container_key="Disks",
            item_key="Disk",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def get_disk(
        self,
        disk_id: str,
        *,
        region: str | None = None,
        **kwargs: Any,
    ) -> ReturnResponse:
        """Get a single ECS disk by id.

        Args:
            disk_id: Disk id.
            region: Optional region override.
            **kwargs: Additional request parameters.

        Returns:
            ReturnResponse: ``data`` is disk dict or ``None`` when missing.
        """
        request_kwargs = dict(kwargs)
        request_kwargs.setdefault("disk_ids", json.dumps([disk_id]))
        response = self.list_disks(region=region, page_size=1, **request_kwargs)
        disks = response.data if isinstance(response.data, list) else []
        return ReturnResponse(code=response.code, msg=response.msg, data=disks[0] if disks else None)

    def list_snapshots(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS snapshots.

        Args:
            region: Optional region override.
            page_size: Number of snapshots per page.
            **kwargs: Pass-through params for ``DescribeSnapshotsRequest``.

        Returns:
            ReturnResponse: ``data`` is list of snapshot dicts.
        """
        return self._list_paginated(
            action="ecs_list_snapshots",
            request_cls=ecs_models.DescribeSnapshotsRequest,
            sdk_method_name="describe_snapshots",
            container_key="Snapshots",
            item_key="Snapshot",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_auto_snapshot_policies(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List automatic snapshot policies.

        Args:
            region: Optional region override.
            page_size: Number of policies per page.
            **kwargs: Pass-through params for ``DescribeAutoSnapshotPolicyExRequest``.

        Returns:
            ReturnResponse: ``data`` is list of policy dicts.
        """
        return self._list_paginated(
            action="ecs_list_auto_snapshot_policies",
            request_cls=ecs_models.DescribeAutoSnapshotPolicyExRequest,
            sdk_method_name="describe_auto_snapshot_policy_ex",
            container_key="AutoSnapshotPolicies",
            item_key="AutoSnapshotPolicy",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_auto_snapshot_policy_associations(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List automatic snapshot policy associations.

        Args:
            region: Optional region override.
            page_size: Number of associations per page.
            **kwargs: Pass-through params for ``DescribeAutoSnapshotPolicyAssociationsRequest``.

        Returns:
            ReturnResponse: ``data`` is list of association dicts.
        """
        return self._list_paginated(
            action="ecs_list_auto_snapshot_policy_associations",
            request_cls=ecs_models.DescribeAutoSnapshotPolicyAssociationsRequest,
            sdk_method_name="describe_auto_snapshot_policy_associations",
            container_key="AutoSnapshotPolicyAssociations",
            item_key="AutoSnapshotPolicyAssociation",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_security_groups(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List security groups.

        Args:
            region: Optional region override.
            page_size: Number of security groups per page.
            **kwargs: Pass-through params for ``DescribeSecurityGroupsRequest``.

        Returns:
            ReturnResponse: ``data`` is list of security group dicts.
        """
        return self._list_paginated(
            action="ecs_list_security_groups",
            request_cls=ecs_models.DescribeSecurityGroupsRequest,
            sdk_method_name="describe_security_groups",
            container_key="SecurityGroups",
            item_key="SecurityGroup",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def get_security_group_attribute(
        self,
        security_group_id: str,
        *,
        region: str | None = None,
        **kwargs: Any,
    ) -> ReturnResponse:
        """Get security group rule details.

        Args:
            security_group_id: Security group id.
            region: Optional region override.
            **kwargs: Additional request parameters.

        Returns:
            ReturnResponse: ``data`` is security group detail dict.
        """
        region_id = region or self._c.cfg.region
        req = ecs_models.DescribeSecurityGroupAttributeRequest(
            region_id=region_id,
            security_group_id=security_group_id,
            **kwargs,
        )
        resp = self._c.call(
            "ecs_get_security_group_attribute",
            lambda: self._c.ecs.describe_security_group_attribute(req),
        )
        body = getattr(resp, "body", None)
        body_map = body.to_map() if hasattr(body, "to_map") else {}
        return ReturnResponse(code=0, msg="success", data=body_map or None)

    def list_network_interfaces(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ENIs.

        Args:
            region: Optional region override.
            page_size: Number of ENIs per page.
            **kwargs: Pass-through params for ``DescribeNetworkInterfacesRequest``.

        Returns:
            ReturnResponse: ``data`` is list of ENI dicts.
        """
        return self._list_paginated(
            action="ecs_list_network_interfaces",
            request_cls=ecs_models.DescribeNetworkInterfacesRequest,
            sdk_method_name="describe_network_interfaces",
            container_key="NetworkInterfaceSets",
            item_key="NetworkInterfaceSet",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_instances_full_status(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS instance full status records.

        Args:
            region: Optional region override.
            page_size: Number of records per page.
            **kwargs: Pass-through params for ``DescribeInstancesFullStatusRequest``.

        Returns:
            ReturnResponse: ``data`` is list of full status dicts.
        """
        return self._list_paginated(
            action="ecs_list_instances_full_status",
            request_cls=ecs_models.DescribeInstancesFullStatusRequest,
            sdk_method_name="describe_instances_full_status",
            container_key="InstanceFullStatusSet",
            item_key="InstanceFullStatusType",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_instance_history_events(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS instance system events.

        Args:
            region: Optional region override.
            page_size: Number of records per page.
            **kwargs: Pass-through params for ``DescribeInstanceHistoryEventsRequest``.

        Returns:
            ReturnResponse: ``data`` is list of event dicts.
        """
        return self._list_paginated(
            action="ecs_list_instance_history_events",
            request_cls=ecs_models.DescribeInstanceHistoryEventsRequest,
            sdk_method_name="describe_instance_history_events",
            container_key="InstanceSystemEventSet",
            item_key="InstanceSystemEventType",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_cloud_assistant_status(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List Cloud Assistant agent status.

        Args:
            region: Optional region override.
            page_size: Number of records per page.
            **kwargs: Pass-through params for ``DescribeCloudAssistantStatusRequest``.

        Returns:
            ReturnResponse: ``data`` is list of agent status dicts.
        """
        return self._list_paginated(
            action="ecs_list_cloud_assistant_status",
            request_cls=ecs_models.DescribeCloudAssistantStatusRequest,
            sdk_method_name="describe_cloud_assistant_status",
            container_key="InstanceCloudAssistantStatusSet",
            item_key="InstanceCloudAssistantStatus",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_images(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS images.

        Args:
            region: Optional region override.
            page_size: Number of images per page.
            **kwargs: Pass-through params for ``DescribeImagesRequest``.

        Returns:
            ReturnResponse: ``data`` is list of image dicts.
        """
        return self._list_paginated(
            action="ecs_list_images",
            request_cls=ecs_models.DescribeImagesRequest,
            sdk_method_name="describe_images",
            container_key="Images",
            item_key="Image",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def list_key_pairs(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List ECS key pairs.

        Args:
            region: Optional region override.
            page_size: Number of key pairs per page.
            **kwargs: Pass-through params for ``DescribeKeyPairsRequest``.

        Returns:
            ReturnResponse: ``data`` is list of key pair dicts.
        """
        return self._list_paginated(
            action="ecs_list_key_pairs",
            request_cls=ecs_models.DescribeKeyPairsRequest,
            sdk_method_name="describe_key_pairs",
            container_key="KeyPairs",
            item_key="KeyPair",
            region=region,
            page_size=page_size,
            **kwargs,
        )

    def count_unassociated_eip_addresses(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        status: str = "Available",
        **kwargs: Any,
    ) -> ReturnResponse:
        """Count unassociated EIP addresses.

        Args:
            region: Optional region override.
            page_size: Number of resources per page.
            status: EIP status filter.
            **kwargs: Pass-through params for ``DescribeEipAddressesRequest``.

        Returns:
            ReturnResponse: ``data`` contains ``count``.
        """
        region_id = region or self._c.cfg.region
        page_number = 1
        count = 0
        request_kwargs = dict(kwargs)
        request_kwargs.setdefault("status", status)

        while True:
            req = ecs_models.DescribeEipAddressesRequest(
                region_id=region_id,
                page_size=page_size,
                page_number=page_number,
                **request_kwargs,
            )
            resp = self._c.call(
                "ecs_count_unassociated_eip_addresses",
                lambda: self._c.ecs.describe_eip_addresses(req),
            )
            body = getattr(resp, "body", None)
            body_map = body.to_map() if hasattr(body, "to_map") else {}

            eips = self._extract_eip_addresses(body_map)
            if not eips:
                break

            count += sum(1 for item in eips if isinstance(item, dict) and not item.get("InstanceId"))

            total_count = int(body_map.get("TotalCount", 0) or 0)
            if page_number * page_size >= total_count:
                break
            page_number += 1

        return ReturnResponse(code=0, msg="success", data={"count": count})

    def count_unattached_disks(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        status: str = "Available",
        **kwargs: Any,
    ) -> ReturnResponse:
        """Count unattached disks.

        Args:
            region: Optional region override.
            page_size: Number of resources per page.
            status: Disk status filter.
            **kwargs: Pass-through params for ``DescribeDisksRequest``.

        Returns:
            ReturnResponse: ``data`` contains ``count``.
        """
        region_id = region or self._c.cfg.region
        page_number = 1
        count = 0
        request_kwargs = dict(kwargs)
        request_kwargs.setdefault("status", status)

        while True:
            req = ecs_models.DescribeDisksRequest(
                region_id=region_id,
                page_size=page_size,
                page_number=page_number,
                **request_kwargs,
            )
            resp = self._c.call(
                "ecs_count_unattached_disks",
                lambda: self._c.ecs.describe_disks(req),
            )
            body = getattr(resp, "body", None)
            body_map = body.to_map() if hasattr(body, "to_map") else {}

            disks = self._extract_disks(body_map)
            if not disks:
                break

            count += sum(1 for item in disks if isinstance(item, dict) and not item.get("InstanceId"))

            total_count = int(body_map.get("TotalCount", 0) or 0)
            if page_number * page_size >= total_count:
                break
            page_number += 1

        return ReturnResponse(code=0, msg="success", data={"count": count})



#!/usr/bin/env python3

import requests
from .utils.response import ReturnResponse


class Notion:
    '''
    Notion API 简单封装类
    '''
    def __init__(self, token: str, proxy: str = None, timeout: int=10):
        """
        初始化对象。

        Args:
            token: token 参数。
            proxy: proxy 参数。
            timeout: timeout 参数。
        """
        self.token = token
        self.headers = {
            "accept": "application/json",
            "Notion-Version": "2022-06-28",
            "content-type": "application/json",
            "Authorization": f"Bearer {self.token}"
        }
        self.timeout = timeout
        self.proxy = proxy
        self.session = requests.Session()
        self.session.headers.update(self.headers)
        self.session.proxies.update({"http": self.proxy, "https": self.proxy})
        self.base_url = "https://api.notion.com/v1"
    
    def database_create(self, page_id: str, name: str = "test databse") -> ReturnResponse:
        """
        在指定的 page 下创建一个简单的数据库
        
        Args:
            page_id: 父页面的 ID（不需要带横杠）
            database_name: 数据库名称
            
        Returns:
            dict: API 响应结果
        """
        url = f"{self.base_url}/databases"

        payload = {
            "parent": {
                "type": "page_id",
                "page_id": page_id
            },
            "title": [
                {
                    "type": "text",
                    "text": {
                        "content": name
                    }
                }
            ],
            "properties": {
                "名称": {
                    "title": {}
                }
            }
        }
        
        response = self.session.post(url, json=payload, timeout=self.timeout)
        
        if response.status_code == 200:
            return ReturnResponse(code=0, msg=f"数据库创建成功: {name}", data=response.json())
        else:
            return ReturnResponse(code=1, msg=f"数据库创建失败: {response.status_code}", data=response.text)

    def database_update(self, db_id: str, title: str = None, description: str = None, icon: dict = None, cover: dict = None) -> ReturnResponse:
        """
        更新数据库属性
        
        Args:
            db_id: 数据库 ID
            title: 数据库标题
            description: 数据库描述
            icon: 数据库图标
            cover: 数据库封面
            
        Returns:
            ReturnResponse: 更新结果
        """
        url = f"{self.base_url}/databases/{db_id}"
        
        payload = {}
        
        # 构建更新字段
        if title is not None:
            payload["title"] = [
                {
                    "type": "text",
                    "text": {
                        "content": title
                    }
                }
            ]
        
        if description is not None:
            payload["description"] = [
                {
                    "type": "text", 
                    "text": {
                        "content": description
                    }
                }
            ]
            
        if icon is not None:
            payload["icon"] = icon
            
        if cover is not None:
            payload["cover"] = cover
        
        # 如果没有提供任何更新字段，返回错误
        if not payload:
            return ReturnResponse(code=1, msg="至少需要提供一个更新字段", data=None)
        
        response = self.session.patch(url, json=payload, timeout=self.timeout)
        
        if response.status_code == 200:
            return ReturnResponse(code=0, msg="数据库更新成功", data=response.json())
        else:
            return ReturnResponse(code=1, msg=f"数据库更新失败: {response.status_code}", data=response.text)

    def page_create(self, db_id: str, properties: dict = None) -> ReturnResponse:
        """
        在数据库中创建新页面（添加数据）
        
        Args:
            db_id: 数据库 ID
            properties: 页面属性字典，键为属性名，值为属性值
            
        Returns:
            ReturnResponse: 创建结果
        """
        url = f"{self.base_url}/pages"
        
        # 默认属性
        if properties is None:
            properties = {}
        
        # 构建请求体
        payload = {
            "parent": {
                "type": "database_id",
                "database_id": db_id
            },
            "properties": {}
        }
        
        # 处理属性
        for key, value in properties.items():
            payload["properties"][key] = self._format_property_value(key, value)
        
        response = self.session.post(url, json=payload, timeout=self.timeout)
        
        if response.status_code == 200:
            return ReturnResponse(code=0, msg="页面创建成功", data=response.json())
        else:
            return ReturnResponse(code=1, msg=f"页面创建失败: {response.status_code}", data=response.text)

    def page_update(self, page_id: str, properties: dict = None) -> ReturnResponse:
        """
        更新页面属性
        
        Args:
            page_id: 页面 ID
            properties: 页面属性字典
            
        Returns:
            ReturnResponse: 更新结果
        """
        url = f"{self.base_url}/pages/{page_id}"
        
        if properties is None:
            properties = {}
        
        # 构建请求体
        payload = {"properties": {}}
        
        # 处理属性
        for key, value in properties.items():
            payload["properties"][key] = self._format_property_value(key, value)
        
        response = self.session.patch(url, json=payload, timeout=self.timeout)
        
        if response.status_code == 200:
            return ReturnResponse(code=0, msg="页面更新成功", data=response.json())
        else:
            return ReturnResponse(code=1, msg=f"页面更新失败: {response.status_code}", data=response.text)

    def page_upsert(self, db_id: str, unique_field: str, unique_value: str, properties: dict = None, content: list = None) -> ReturnResponse:
        """
        智能创建或更新页面（Upsert操作）
        
        根据唯一字段查找页面：
        - 如果找到匹配的页面，则更新该页面的属性
        - 如果未找到匹配的页面，则创建新页面
        
        支持的属性类型：
        - 标题属性 (title): 字段名包含 name/title/名称/标题
        - 文本属性 (rich_text): 普通文本字段
        - 复选框属性 (checkbox): 布尔值
        - 数字属性 (number): 数字值
        - 多选属性 (multi_select): 字符串列表
        - 单选属性 (select): 字典格式 {"name": "选项名"}
        - 日期属性 (date): 字典格式 {"start": "2024-01-01"}
        - URL属性 (url): 字典格式 {"url": "https://example.com"}
        - 邮箱属性 (email): 字典格式 {"email": "test@example.com"}
        - 电话号码属性 (phone_number): 字典格式 {"phone_number": "123-456-7890"}
        - 状态属性 (status): 字典格式 {"name": "状态名"}
        - 关系属性 (relation): 支持多种格式
          * 直接ID: [{"id": "页面ID"}]
          * 便捷格式: [{"title": "页面标题", "database_id": "数据库ID"}]
          * 混合格式: [{"id": "页面ID1"}, {"title": "页面标题", "database_id": "数据库ID"}]
        - 人员属性 (people): 列表格式 [{"id": "用户ID"}] 或 [{"email": "邮箱"}]
        
        Args:
            db_id: 数据库 ID
            unique_field: 用于查找的唯一字段名（如"名称"、"Name"等）
            unique_value: 唯一字段的值，用于查找匹配的页面
            properties: 页面属性字典，包含要设置或更新的属性
            content: 页面内容列表，包含要添加的内容块
            
        Returns:
            ReturnResponse: 创建或更新结果
            - code=0: 操作成功
            - code=1: 操作失败，查看 msg 和 data 获取错误信息
            
        Example:
            # 第一次调用会创建新页面
            result = notion.page_upsert(
                db_id="database_id",
                unique_field="Name",
                unique_value="项目A",
                properties={
                    "Name": "项目A",
                    "Status": {"name": "进行中"},
                    "Priority": 1,
                    "Tags": ["重要", "紧急"],
                    # 关系字段便捷用法
                    "RelatedPages": [
                        {"title": "相关项目1", "database_id": "related_db_id"},
                        {"title": "相关项目2", "database_id": "related_db_id"}
                    ]
                },
                # 页面内容
                content=[
                    {"type": "heading_1", "text": "项目概述"},
                    {"type": "paragraph", "text": "这是项目的详细描述"},
                    {"type": "to_do", "text": "完成需求分析", "checked": True},
                    {"type": "to_do", "text": "设计系统架构", "checked": False}
                ]
            )
            
            # 第二次调用会更新现有页面
            result = notion.page_upsert(
                db_id="database_id",
                unique_field="Name", 
                unique_value="项目A",  # 相同的唯一值
                properties={
                    "Name": "项目A",
                    "Status": {"name": "已完成"},  # 更新状态
                    "Priority": 2,  # 更新优先级
                    # 混合关系字段格式
                    "RelatedPages": [
                        {"id": "existing_page_id"},  # 直接ID
                        {"title": "新相关项目", "database_id": "related_db_id"}  # 便捷格式
                    ]
                },
                # 更新页面内容
                content=[
                    {"type": "heading_2", "text": "更新内容"},
                    {"type": "paragraph", "text": "项目状态已更新"},
                    {"type": "quote", "text": "项目进展顺利"}
                ]
            )
        """
        if properties is None:
            properties = {}
        if content is None:
            content = []
        
        # 1. 先查询数据库，看是否存在相同记录
        query_url = f"{self.base_url}/databases/{db_id}/query"
        query_payload = {
            "filter": {
                "property": unique_field,
                "title": {
                    "equals": unique_value
                }
            }
        }
        
        query_response = self.session.post(query_url, json=query_payload, timeout=self.timeout)
        
        if query_response.status_code == 200:
            results = query_response.json().get("results", [])
            
            if results:
                # 找到记录，执行更新
                page_id = results[0]["id"]
                
                # 更新页面属性
                update_result = self.page_update(page_id, properties)
                
                # 如果有内容需要添加，则添加内容
                if content and update_result.code == 0:
                    content_result = self.page_add_content(page_id, content)
                    if content_result.code == 0:
                        return content_result
                    else:
                        return update_result
                else:
                    return update_result
            else:
                # 未找到记录，执行创建
                # 确保唯一字段包含在属性中
                if unique_field not in properties:
                    properties[unique_field] = unique_value
                
                # 创建页面
                create_result = self.page_create(db_id, properties)
                
                # 如果有内容需要添加，则添加内容
                if content and create_result.code == 0:
                    page_id = create_result.data["id"]
                    content_result = self.page_add_content(page_id, content)
                    if content_result.code == 0:
                        return content_result
                    else:
                        return create_result
                else:
                    return create_result
        else:
            # 查询失败，直接尝试创建
            if unique_field not in properties:
                properties[unique_field] = unique_value
            
            # 创建页面
            create_result = self.page_create(db_id, properties)
            
            # 如果有内容需要添加，则添加内容
            if content and create_result.code == 0:
                page_id = create_result.data["id"]
                content_result = self.page_add_content(page_id, content)
                if content_result.code == 0:
                    return content_result
                else:
                    return create_result
            else:
                return create_result

    def _find_page_by_title(self, db_id: str, title: str) -> str:
        """
        根据标题在数据库中查找页面ID
        
        Args:
            db_id: 数据库ID
            title: 页面标题
            
        Returns:
            str: 页面ID，如果未找到返回None
        """
        query_url = f"{self.base_url}/databases/{db_id}/query"
        query_payload = {
            "filter": {
                "property": "Name",  # 假设标题字段名为"Name"
                "title": {
                    "equals": title
                }
            }
        }
        
        response = self.session.post(query_url, json=query_payload, timeout=self.timeout)
        
        if response.status_code == 200:
            results = response.json().get("results", [])
            if results:
                return results[0]["id"]
        
        return None

    def _format_relation_value(self, value) -> list:
        """
        格式化关系属性值，支持多种输入格式
        
        Args:
            value: 关系值，支持以下格式：
                - 字符串: 页面标题，需要指定database_id
                - 字典: {"title": "页面标题", "database_id": "数据库ID"}
                - 列表: [{"title": "标题1", "database_id": "数据库ID1"}, ...]
                - 列表: [{"id": "页面ID1"}, {"id": "页面ID2"}]
                
        Returns:
            list: 格式化后的关系值列表
        """
        if isinstance(value, str):
            # 字符串格式，需要database_id
            raise ValueError("字符串格式需要同时提供database_id，请使用字典格式：{'title': '页面标题', 'database_id': '数据库ID'}")
        
        elif isinstance(value, dict):
            if "id" in value:
                # 直接提供页面ID
                return [{"id": value["id"]}]
            elif "title" in value and "database_id" in value:
                # 提供标题和数据库ID，需要查找页面ID
                page_id = self._find_page_by_title(value["database_id"], value["title"])
                if page_id:
                    return [{"id": page_id}]
                else:
                    raise ValueError(f"未找到标题为 '{value['title']}' 的页面")
            else:
                raise ValueError("字典格式必须包含 'id' 或 ('title' 和 'database_id')")
        
        elif isinstance(value, list):
            relation_result = []
            for item in value:
                if isinstance(item, str):
                    raise ValueError("列表中的字符串需要提供database_id")
                elif isinstance(item, dict):
                    if "id" in item:
                        relation_result.append({"id": item["id"]})
                    elif "title" in item and "database_id" in item:
                        page_id = self._find_page_by_title(item["database_id"], item["title"])
                        if page_id:
                            relation_result.append({"id": page_id})
                        else:
                            raise ValueError(f"未找到标题为 '{item['title']}' 的页面")
                    else:
                        raise ValueError("列表项必须包含 'id' 或 ('title' 和 'database_id')")
                else:
                    raise ValueError("列表项必须是字典格式")
            return relation_result
        
        else:
            raise ValueError("不支持的关系值格式")

    def _format_property_value(self, key: str, value) -> dict:
        """
        根据属性类型格式化属性值
        
        Args:
            key: 属性名
            value: 属性值
            
        Returns:
            dict: 格式化后的属性值
        """
        if isinstance(value, str):
            # 根据字段名判断属性类型
            if key.lower() in ['name', 'title', '名称', '标题']:
                # 标题属性
                return {
                    "title": [
                        {
                            "type": "text",
                            "text": {
                                "content": value
                            }
                        }
                    ]
                }
            else:
                # 文本属性 (rich_text)
                return {
                    "rich_text": [
                        {
                            "type": "text",
                            "text": {
                                "content": value
                            }
                        }
                    ]
                }
        elif isinstance(value, bool):
            # 复选框属性
            return {"checkbox": value}
        elif isinstance(value, (int, float)):
            # 数字属性
            return {"number": value}
        elif isinstance(value, list):
            # 列表类型，需要进一步判断
            if not value:
                # 空列表，默认作为多选处理
                return {"multi_select": []}
            
            # 根据第一个元素的类型判断
            first_item = value[0]
            if isinstance(first_item, str):
                # 多选属性
                return {
                    "multi_select": [
                        {"name": item} for item in value
                    ]
                }
            elif isinstance(first_item, dict):
                # 复杂对象列表
                if "name" in first_item:
                    # 多选属性
                    return {"multi_select": value}
                elif "id" in first_item or "title" in first_item:
                    # 关系属性
                    try:
                        return {"relation": self._format_relation_value(value)}
                    except ValueError:
                        return {"relation": value}  # 回退到原始值
                elif "email" in first_item:
                    # 人员属性
                    return {"people": value}
                elif "object" in first_item and first_item["object"] == "user":
                    # 人员属性（用户对象格式）
                    return {"people": value}
            return {"multi_select": [{"name": str(item)} for item in value]}
        elif isinstance(value, dict):
            # 字典类型
            if "start" in value:
                # 日期属性
                return {"date": value}
            elif "name" in value:
                # 单选属性
                return {"select": value}
            elif "url" in value:
                # URL属性
                return {"url": value["url"]}
            elif "email" in value:
                # 邮箱属性
                return {"email": value["email"]}
            elif "phone_number" in value:
                # 电话号码属性
                return {"phone_number": value["phone_number"]}
            elif "status" in value:
                # 状态属性
                return {"status": value}
            elif "id" in value or "title" in value:
                # 关系属性
                try:
                    return {"relation": self._format_relation_value(value)}
                except ValueError:
                    return {"relation": value}  # 回退到原始值
            else:
                # 默认作为单选处理
                return {"select": value}
        else:
            # 其他类型，转换为字符串处理
            return {
                "rich_text": [
                    {
                        "type": "text",
                        "text": {
                            "content": str(value)
                        }
                    }
                ]
            }

    def page_add_content(self, page_id: str, content: list) -> ReturnResponse:
        """
        向页面添加内容块
        
        Args:
            page_id: 页面 ID
            content: 内容块列表，每个元素是一个内容块
            
        Returns:
            ReturnResponse: 添加结果
        """
        url = f"{self.base_url}/blocks/{page_id}/children"
        
        # 格式化内容块
        formatted_blocks = []
        for block in content:
            formatted_blocks.append(self._format_content_block(block))
        
        payload = {"children": formatted_blocks}
        
        response = self.session.patch(url, json=payload, timeout=self.timeout)
        
        if response.status_code == 200:
            return ReturnResponse(code=0, msg="页面内容添加成功", data=response.json())
        else:
            return ReturnResponse(code=1, msg=f"页面内容添加失败: {response.status_code}", data=response.text)

    def _format_content_block(self, block: dict) -> dict:
        """
        格式化内容块
        
        Args:
            block: 内容块字典，格式如：
                {
                    "type": "paragraph",
                    "text": "这是段落文本"
                }
                或
                {
                    "type": "heading_1",
                    "text": "这是一级标题"
                }
                
        Returns:
            dict: 格式化后的内容块
        """
        block_type = block.get("type", "paragraph")
        text = block.get("text", "")
        
        # 基础文本内容
        text_content = {
            "type": "text",
            "text": {
                "content": text
            }
        }
        
        # 根据类型返回不同的块结构
        if block_type == "paragraph":
            return {
                "type": "paragraph",
                "paragraph": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "heading_1":
            return {
                "type": "heading_1",
                "heading_1": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "heading_2":
            return {
                "type": "heading_2",
                "heading_2": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "heading_3":
            return {
                "type": "heading_3",
                "heading_3": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "bulleted_list_item":
            return {
                "type": "bulleted_list_item",
                "bulleted_list_item": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "numbered_list_item":
            return {
                "type": "numbered_list_item",
                "numbered_list_item": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "to_do":
            checked = block.get("checked", False)
            return {
                "type": "to_do",
                "to_do": {
                    "rich_text": [text_content],
                    "checked": checked
                }
            }
        elif block_type == "quote":
            return {
                "type": "quote",
                "quote": {
                    "rich_text": [text_content]
                }
            }
        elif block_type == "code":
            language = block.get("language", "plain text")
            return {
                "type": "code",
                "code": {
                    "rich_text": [text_content],
                    "language": language
                }
            }
        else:
            # 默认作为段落处理
            return {
                "type": "paragraph",
                "paragraph": {
                    "rich_text": [text_content]
                }
            }

if __name__ == "__main__":
    # 使用示例
    notion = Notion(token="your_integration_token_here")
    database_id = "your_database_id_here"
    
    # 使用智能创建/更新方法
    # 第一次调用会创建新记录
    result = notion.page_upsert(
        db_id=database_id,
        unique_field="名称",  # 用于查找的唯一字段
        unique_value="测试记录",  # 唯一字段的值
        properties={
            "名称": "测试记录",
            "状态": ["进行中"],
            "完成": False,
            "优先级": 1
        }
    )
    
    # 第二次调用会更新现有记录
    result = notion.page_upsert(
        db_id=database_id,
        unique_field="名称",
        unique_value="测试记录",  # 相同的唯一值
        properties={
            "名称": "测试记录",
            "状态": ["已完成"],  # 更新状态
            "完成": True,        # 更新完成状态
            "优先级": 2          # 更新优先级
        }
    )
    
    # 使用不同的唯一值会创建新记录
    result = notion.page_upsert(
        db_id=database_id,
        unique_field="名称",
        unique_value="新记录",
        properties={
            "名称": "新记录",
            "状态": ["待处理"],
            "完成": False,
            "优先级": 3
        }
    )



#!/usr/bin/env python3

import json
from pathlib import Path

from pytbox.utils.load_config import load_config_by_file


def test_load_jsonfile_placeholder(tmp_path: Path) -> None:
    """`load_config_by_file` should resolve `jsonfile,` placeholders."""
    json_path = tmp_path / "values.json"
    json_path.write_text(json.dumps({"json_test": "value-from-json"}), encoding="utf-8")

    toml_path = tmp_path / "config.toml"
    toml_path.write_text('json_test = "jsonfile,json_test"\n', encoding="utf-8")

    config = load_config_by_file(path=str(toml_path), jsonfile=str(json_path))
    assert config["json_test"] == "value-from-json"



"""Network CLI commands."""

from __future__ import annotations

from typing import Any, Dict

import click

from pytbox.network.config_loader import load_backup_config
from pytbox.network.device_backup import NetworkBackupService


def _has_direct_args(
    device_os: str | None,
    ip: str | None,
    protocol: str | None,
    username: str | None,
    password: str | None,
) -> bool:
    """Check whether direct device args are provided.

    Args:
        device_os: Device OS.
        ip: Device IP.
        protocol: Device protocol.
        username: Device username.
        password: Device password.

    Returns:
        bool: Whether any direct arg exists.
    """
    return any([device_os, ip, protocol, username, password])


def _build_merged_direct_device(
    base_config: Dict[str, Any],
    device_os: str,
    ip: str,
    protocol: str,
    username: str,
    password: str,
) -> Dict[str, Any]:
    """Build single direct-mode device and merge with config.

    Args:
        base_config: Config loaded from file or defaults.
        device_os: Device OS.
        ip: Device IP.
        protocol: Device protocol.
        username: Device username.
        password: Device password.

    Returns:
        dict[str, Any]: Merged device config.
    """
    base_device: Dict[str, Any] = {}
    devices = base_config.get("devices", [])
    if devices and isinstance(devices, list) and isinstance(devices[0], dict):
        base_device = devices[0].copy()

    overrides = {
        "os": device_os.lower(),
        "ip": ip,
        "protocol": protocol.lower(),
        "username": username,
        "password": password,
    }
    base_device.update(overrides)
    return base_device


@click.group()
def network_group() -> None:
    """Network automation commands."""


@network_group.command("backup")
@click.option(
    "--config",
    "config_path",
    type=click.Path(exists=True, dir_okay=False),
    help="配置文件路径 (yaml/yml/json/toml)",
)
@click.option(
    "--os",
    "device_os",
    type=click.Choice(["cisco", "huawei", "h3c", "ruijie"], case_sensitive=False),
    help="设备厂商",
)
@click.option("--ip", "ip", type=str, help="设备 IP 地址")
@click.option(
    "--protocol",
    "protocol",
    type=click.Choice(["ssh", "telnet"], case_sensitive=False),
    help="连接协议",
)
@click.option("--username", "username", type=str, help="登录用户名")
@click.option("--password", "password", type=str, help="登录密码")
@click.option("--output-dir", "output_dir", type=str, default=None, help="备份输出目录")
def backup_command(
    config_path: str | None,
    device_os: str | None,
    ip: str | None,
    protocol: str | None,
    username: str | None,
    password: str | None,
    output_dir: str | None,
) -> None:
    """Backup device running configuration.

    Args:
        config_path: Config file path.
        device_os: Device OS.
        ip: Device IP.
        protocol: Connection protocol.
        username: Login username.
        password: Login password.
        output_dir: Optional output directory override.
    """
    direct_mode = _has_direct_args(device_os, ip, protocol, username, password)
    if not config_path and not direct_mode:
        raise click.UsageError("请提供 --config 或直连参数(--os/--ip/--protocol/--username)")

    merged_config: Dict[str, Any] = {
        "output_dir": "./backups/network",
        "timeout": 10,
        "retries": 3,
        "devices": [],
    }
    if config_path:
        loaded = load_backup_config(config_path)
        if loaded.code != 0:
            raise click.ClickException(loaded.msg)
        if not isinstance(loaded.data, dict):
            raise click.ClickException("配置格式无效")
        merged_config = loaded.data

    if direct_mode:
        required_direct_fields = {
            "os": device_os,
            "ip": ip,
            "protocol": protocol,
            "username": username,
        }
        missing_direct = [
            field_name
            for field_name, field_value in required_direct_fields.items()
            if not field_value
        ]
        if missing_direct:
            raise click.UsageError(
                f"直连模式缺少参数: {', '.join(missing_direct)}"
            )
        base_password = ""
        base_devices = merged_config.get("devices", [])
        if (
            isinstance(base_devices, list)
            and base_devices
            and isinstance(base_devices[0], dict)
        ):
            base_password = str(base_devices[0].get("password") or "")

        password_value = password or base_password
        if not password_value:
            password_value = click.prompt("Password", hide_input=True)
        merged_device = _build_merged_direct_device(
            base_config=merged_config,
            device_os=str(device_os),
            ip=str(ip),
            protocol=str(protocol),
            username=str(username),
            password=str(password_value),
        )
        merged_config["devices"] = [merged_device]
    elif not merged_config.get("devices"):
        raise click.UsageError("配置文件中 devices 不能为空")

    if output_dir:
        merged_config["output_dir"] = output_dir

    service = NetworkBackupService()
    response = service.backup_devices(merged_config)
    summary = response.data if isinstance(response.data, dict) else {}
    success_count = int(summary.get("success_count", 0))
    failed_count = int(summary.get("failed_count", 0))

    click.echo(f"Backup finished. success={success_count}, failed={failed_count}")
    if response.code != 0:
        click.echo(f"Backup error: {response.msg}", err=True)
        failed_items = summary.get("failed", []) if isinstance(summary, dict) else []
        if isinstance(failed_items, list):
            for item in failed_items:
                if not isinstance(item, dict):
                    continue
                ip_value = str(item.get("ip", "unknown"))
                os_value = str(item.get("os", "unknown"))
                reason_value = str(item.get("reason", "unknown error"))
                click.echo(
                    f"Failed target ip={ip_value} os={os_value} reason={reason_value}",
                    err=True,
                )
        raise SystemExit(1)



"""Aliyun SLB resource operations."""

from __future__ import annotations

from typing import Any

from alibabacloud_slb20140515 import models as slb_models
from alibabacloud_tea_util import models as util_models

from ...schemas.response import ReturnResponse
from ._helpers import body_to_map, extract_collection, extract_total, failure_response


class SLBResource:
    """Aliyun SLB read-only resource wrapper."""

    def __init__(self, client: Any) -> None:
        """Initialize resource.

        Args:
            client: AliyunClient instance.
        """
        self._c = client

    def list_load_balancers(
        self,
        *,
        region: str | None = None,
        page_size: int = 50,
        **kwargs: Any,
    ) -> ReturnResponse:
        """List CLB load balancers."""
        try:
            region_id = region or self._c.cfg.region
            page_number = 1
            load_balancers_all: list[dict[str, Any]] = []
            runtime = util_models.RuntimeOptions()

            while True:
                request = slb_models.DescribeLoadBalancersRequest(
                    region_id=region_id,
                    page_number=page_number,
                    page_size=page_size,
                    **kwargs,
                )
                response = self._c.call(
                    "slb_list_load_balancers",
                    lambda: self._c.slb.describe_load_balancers_with_options(request, runtime),
                )
                body_map = body_to_map(response)
                items = extract_collection(
                    body_map,
                    container_key="LoadBalancers",
                    item_key="LoadBalancer",
                )
                if not items:
                    break
                load_balancers_all.extend(items)

                total_count = extract_total(body_map, "TotalCount")
                if total_count is not None:
                    if page_number * page_size >= total_count:
                        break
                elif len(items) < page_size:
                    break
                page_number += 1

            return ReturnResponse(code=0, msg="success", data=load_balancers_all)
        except Exception as error:  # noqa: BLE001
            return failure_response(error)



#!/usr/bin/env python3

from typing import Literal, Type

import requests

from .utils.response import ReturnResponse


class Mingdao:
    '''
    _summary_
    '''
    def __init__(self, app_key: str=None, sign: str=None, timeout: int=5):
        """
        初始化对象。

        Args:
            app_key: app_key 参数。
            sign: sign 参数。
            timeout: timeout 参数。
        """
        self.base_url = "https://api.mingdao.com"
        self.headers = {
            'Content-Type': 'application/json;charset=UTF-8',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36'
        }
        self.timeout = timeout
        self.app_key = app_key
        self.sign = sign

    def _build_api_request(self, api_url: str, method: Literal['GET', 'POST'], params: dict=None, body: dict=None, api_version: Literal['v1', 'v2']='v2'):
        """
        执行 build api request 相关逻辑。

        Args:
            api_url: api_url 参数。
            method: method 参数。
            params: params 参数。
            body: body 参数。
            api_version: api_version 参数。

        Returns:
            Any: 返回值。
        """
        body['appKey'] = self.app_key
        body['sign'] = self.sign
        if not api_url.startswith('/'):
            url = f'{self.base_url}/{api_version}/{api_url}'
        else:
            url = f'{self.base_url}/{api_version}{api_url}'
        params = {
            "appKey": self.app_key,
            "sign": self.sign,
        }
        return requests.request(method, url, params=params, headers=self.headers, json=body, timeout=self.timeout)
    
    def get_app_info(self) -> ReturnResponse:
        '''
        _summary_

        Returns:
            ReturnResponse: _description_
        '''
        r = self._build_api_request(api_url='/open/app/get', method='GET', body={}, api_version='v1')
        return ReturnResponse(code=0, msg='获取应用信息成功', data=r.json())
    
    def get_work_sheet_info(self, worksheet_id: str=None, table_name: str=None, worksheet_name: str=None):
        """
        获取work sheet info。

        Args:
            worksheet_id: 资源 ID。
            table_name: table_name 参数。
            worksheet_name: worksheet_name 参数。

        Returns:
            Any: 返回值。
        """
        if worksheet_name:
            worksheet_id = self.get_work_sheet_id_by_name(table_name=table_name, worksheet_name=worksheet_name)
        
        r = self._build_api_request(
            api_url='open/worksheet/getWorksheetInfo',
            method='POST',
            body={
                "worksheetId": worksheet_id,
            }
        )
        return r.json()
    
    def get_project_info(self, worksheet_id: str, keywords: str):
        """
        获取project info。

        Args:
            worksheet_id: 资源 ID。
            keywords: keywords 参数。

        Returns:
            Any: 返回值。
        """
        r = self._build_api_request(
            api_url='open/worksheet/getFilterRows',
            method='POST',
            body={
                "pageIndex": 1,
                "pageSize": 100,
                "worksheetId": worksheet_id,
                "keyWords": keywords,
            }
        )
        return r.json()
    
    def get_work_sheet_id_by_name(self, table_name: str, worksheet_name: str, child_section: bool=False):
        """
        获取work sheet id by name。

        Args:
            table_name: table_name 参数。
            worksheet_name: worksheet_name 参数。
            child_section: child_section 参数。

        Returns:
            Any: 返回值。
        """
        r = self.get_app_info()        
        for i in r.data['data']['sections']:
            if table_name == i['name']:
                if child_section:
                    for child in i['childSections'][0]['items']:
                        if child['name'] == worksheet_name:
                            return child['id']
                else:
                    for item in i['items']:
                        if item['name'] == worksheet_name:
                            return item['id']

    def get_control_id(self, table_name: str=None, worksheet_name: str=None, control_name: str=None):
        """
        获取control id。

        Args:
            table_name: table_name 参数。
            worksheet_name: worksheet_name 参数。
            control_name: control_name 参数。

        Returns:
            Any: 返回值。
        """
        r = self.get_work_sheet_info(table_name=table_name, worksheet_name=worksheet_name)
        for control in r['data']['controls']:
            if control['controlName'] == control_name:
                return control['controlId']
        return None
    
    def get_value(self, table_name: str=None, worksheet_name: str=None, control_name: str=None, value_name: str=None):
        """
        获取value。

        Args:
            table_name: table_name 参数。
            worksheet_name: worksheet_name 参数。
            control_name: control_name 参数。
            value_name: value_name 参数。

        Returns:
            Any: 返回值。
        """
        control_id = self.get_control_id(table_name=table_name, worksheet_name=worksheet_name, control_name=control_name)
  
        r = self._build_api_request(
            api_url='open/worksheet/getFilterRows',
            method='POST',
            body={
                "pageIndex": 1,
                "pageSize": 100,
                "worksheetId": self.get_work_sheet_id_by_name(table_name=table_name, worksheet_name=worksheet_name),
                # "filters": filters
            }
        )
        for row in r.json()['data']['rows']:
            if row[control_id] == value_name:
                return row['rowid']

    def get_work_record(self, 
                        worksheet_id: str=None,
                        project_control_id: str=None,
                        project_value: str=None,
                        complete_date_control_id: str=None,
                        complete_date: Literal['Today', '上个月', 'Last7Day', 'Last30Day']=None,
                        parse_control_id: bool=False,
                        page_size: int=100,
                    ):
        
        """
        获取work record。

        Args:
            worksheet_id: 资源 ID。
            project_control_id: 资源 ID。
            project_value: project_value 参数。
            complete_date_control_id: 资源 ID。
            complete_date: complete_date 参数。
            parse_control_id: 资源 ID。
            page_size: page_size 参数。

        Returns:
            Any: 返回值。
        """
        filters = []
        if project_value:
            filters.append({
                "controlId": project_control_id,
                "dataType": 29,
                "spliceType": 1,
                "filterType": 24,
                "dateRange": 0,
                "dateRangeType": 0,
                "value": "",
                "values": [
                    project_value
                ],
                "minValue": "",
                "maxValue": ""
            })
        
        if complete_date:
            if complete_date == '上个月':
                data_range = 8
            elif complete_date == 'Today':
                data_range = 1
            elif complete_date == 'Last7Day':
                data_range = 21
            else:
                data_range = 1
            
            filters.append({
                "controlId": complete_date_control_id,
                "dataType": 15,
                "spliceType": 1,
                "filterType": 17,
                "dateRange": data_range,
            })
        r = self._build_api_request(
            api_url='open/worksheet/getFilterRows',
            method='POST',
            body={
                "pageIndex": 1,
                "pageSize": page_size,
                "worksheetId": worksheet_id,
                "filters": filters
            }
        )
        return r.json()



from pydantic import BaseModel, Field
from typing import Any, Optional
from .codes import RespCode


class ReturnResponse(BaseModel):
    '''
    _summary_

    Args:
        BaseModel (_type_): _description_

    Returns:
        _type_: _description_
    '''
    code: int = Field(..., description='业务状态码，0=成功，其它=失败或特殊状态')
    msg: str = Field(..., description='状态消息描述')
    data: Optional[Any] = Field(default=None, description='返回的数据')
    
    @classmethod
    def ok(cls, data: Any = None, msg: str = "OK"):
        """
        执行 ok 相关逻辑。

        Args:
            data: data 参数。
            msg: msg 参数。

        Returns:
            Any: 返回值。
        """
        return cls(code=int(RespCode.OK), msg=msg, data=data)
    
    @classmethod
    def fail(cls, code: RespCode, msg: str, data: Any = None):
        """
        执行 fail 相关逻辑。

        Args:
            code: code 参数。
            msg: msg 参数。
            data: data 参数。

        Returns:
            Any: 返回值。
        """
        return cls(code=code, msg=msg, data=data)
    
    @classmethod
    def no_data(cls, msg: str = "No data", data: Any = None):
        """
        执行 no data 相关逻辑。

        Args:
            msg: msg 参数。
            data: data 参数。

        Returns:
            Any: 返回值。
        """
        return cls(code=int(RespCode.NO_DATA), msg=msg, data=data)



#!/usr/bin/env python3

from pytbox.base import mingdao


def test_get_app_info():
    r = mingdao.get_app_info()
    print(r)

def test_get_work_sheet_info():
    r = mingdao.get_work_sheet_info(table_name="首页", worksheet_name="工单中心")
    print(r)

def test_get_project_info():
    r = mingdao.get_project_info(keywords="")
    print(r)

def test_get_work_sheet_id_by_name():
    r = mingdao.get_work_sheet_id_by_name(table_name="首页", worksheet_name="项目信息")
    print(r)

def test_get_control_id():
    r = mingdao.get_control_id(table_name="首页", worksheet_name="工单中心", control_name="完成日期")
    print(r)

def test_get_work_record():
    r = mingdao.get_work_record(
        worksheet_id="64a77d4277649ebe61a9cb45",
        project_control_id="64a822be5760cf37a93d07ff",
        project_value="4b97c271-4997-4230-90be-28d5285f609b",
        complete_date_control_id="64a7e18c77649ebe61aad968",
        complete_date="上个月"
    )
    print(r)
    
if __name__ == '__main__':
    test_get_work_record()



import click
from pytbox.database.vm.backend import HTTPBackend, FileReplayBackend, RecordingBackend
from pytbox.database.vm.client import VictoriaMetricsClient
from pytbox.utils.richutils import RichUtils


rich_utils = RichUtils()


@click.group()
def vm_group():
    """VictoriaMetrics CLI 工具（查询/生成fixture/回放）"""
    pass


def _load_promql_lines(path: str) -> list[str]:
    """
    读取 promql 文件：一行一条
    - 自动忽略空行
    - 忽略以 # 开头的注释
    - 忽略像 '=== PromQL used ===' 这种分隔行
    """
    lines = []
    with open(path, "r", encoding="utf-8") as f:
        for raw in f:
            s = raw.strip()
            if not s:
                continue
            if s.startswith("#"):
                continue
            if s.startswith("===") and s.endswith("==="):
                continue
            lines.append(s)
    return lines


@vm_group.command("record-promql")
@click.option("--url", "-u", required=True, help="VM base url，例如 http://vm:8428")
@click.option("--promql-file", "-f", required=True, type=click.Path(exists=True), help="PromQL 文件：一行一条")
@click.option("--out-dir", default="./fixtures/vm", show_default=True, help="fixture 输出目录")
@click.option("--timeout", default=10, show_default=True, type=int, help="HTTP 超时秒数")
@click.option("--no-overwrite", is_flag=True, help="fixture 已存在则不覆盖（需要后端支持；此处用跳过实现）")
def record_promql(url: str, promql_file: str, out_dir: str, timeout: int, no_overwrite: bool):
    """
    在生产环境执行 promql 文件中的查询，并保存 fixture
    """
    promqls = _load_promql_lines(promql_file)
    if not promqls:
        raise click.UsageError("promql 文件为空或没有有效行")

    http = HTTPBackend(url, timeout=timeout)
    replay = FileReplayBackend(out_dir)

    rich_utils.print(msg=f"Loaded {len(promqls)} promql(s) from: {promql_file}")
    rich_utils.print(msg=f"Saving fixtures to: {out_dir}")

    ok = 0
    fail = 0

    for idx, q in enumerate(promqls, start=1):
        try:
            # 如果你想“存在则不覆盖”，建议在 FileReplayBackend.save_fixture 支持 overwrite=False
            # 这里先简单实现：若 no_overwrite=True 且目标文件存在则跳过
            if no_overwrite:
                p = replay.path_for(q)
                if p.exists():
                    rich_utils.print(msg=f"[{idx}/{len(promqls)}] skip (exists): {p}")
                    ok += 1
                    continue

            raw = http.instant_query(q)
            p = replay.save_fixture(q, raw)  # 里面已写入 _fixture.promql 等信息（如果你按之前增强版实现）
            rich_utils.print(msg=f"[{idx}/{len(promqls)}] saved: {p}")
            ok += 1
        except Exception as e:
            rich_utils.print(msg=f"[{idx}/{len(promqls)}] failed: {q} | {e}")
            fail += 1

    rich_utils.print(msg=f"Done. ok={ok}, fail={fail}")


if __name__ == "__main__":
    vm()


#!/usr/bin/env python3

from ..database.victoriametrics import VictoriaMetrics
from ..lib.load_config import load_config


def ping(config, target):
    """
    探测。

    Args:
        config: config 参数。
        target: target 参数。

    Returns:
        Any: 返回值。
    """
    vm = VictoriaMetrics(url=config['victoriametrics']['url'])
    ping_status = vm.query_ping_status(target=target, last_minutes=config['alert']['ping']['last_minutes'])
    if ping_status == '不通':
        insert_alert(
            event_name=config['alert']['ping']['event_name'],
            event_content=f"{target} {config['alert']['ping']['event_name']}",
            entity_name=target,
            priority=config['alert']['ping']['priority'],
            resolved_query={
                "target": target
            }
        )


if __name__ == "__main__":
    config = load_config()
    ping(config, "10.30.35.38")



#!/usr/bin/env python3

from typing import Literal
from datetime import datetime, timedelta

import requests
from ..utils.response import ReturnResponse
from .mail_detail import MailDetail
from ..utils.timeutils import TimeUtils



class AliMail:
    '''
    _summary_
    '''
    def __init__(self, mail_address: str=None, client_id: str=None, client_secret: str=None, timeout: int=3):
        """
        初始化对象。

        Args:
            mail_address: mail_address 参数。
            client_id: 资源 ID。
            client_secret: client_secret 参数。
            timeout: timeout 参数。
        """
        self.email_address = mail_address
        self.client_id = client_id
        self.client_secret = client_secret
        self.base_url = "https://alimail-cn.aliyuncs.com/v2"
        self.timeout = timeout
        self.headers = {
                "Content-Type": "application/json",
                "Authorization": f"bearer {self._get_access_token_by_request()}"
            }
        self.session = requests.Session()
        self.session.headers.update(self.headers)
        
    
    def _get_access_token_by_request(self):
        '''
        https://mailhelp.aliyun.com/openapi/index.html#/markdown/authorization.md

        Raises:
            ValueError: _description_

        Returns:
            _type_: _description_
        '''
        # 定义接口URL
        interface_url = "https://alimail-cn.aliyuncs.com/oauth2/v2.0/token"
        # 设置请求头，指定内容类型
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        # 准备请求数据
        data = {
            "grant_type": "client_credentials",
            "client_id": self.client_id,
            "client_secret": self.client_secret
        }
        try:
            response = requests.post(interface_url, headers=headers, data=data, timeout=self.timeout)
            response_json = response.json()
            current_time = datetime.now()
            data = {
                'token_type': response_json["token_type"],
                'access_token': response_json["access_token"],
                'expires_in':  response_json["expires_in"],
                'expiration_time': current_time + timedelta(seconds=response_json["expires_in"])
            }
            return data.get("access_token")
        except requests.RequestException as e:
            # 处理请求失败异常
            raise e
        except (KeyError, ValueError) as e:
            # 处理解析响应失败异常
            raise e

    def get_mail_folders(self):
        """
        获取mail folders。

        Returns:
            Any: 返回值。
        """
        response = self.session.get(
            url=f"{self.base_url}/users/{self.email_address}/mailFolders",
            headers=self.headers
        )
        return response.json().get('folders')

    def get_folder_id(self, folder_name: Literal['inbox']='inbox'):
        """
        获取folder id。

        Args:
            folder_name: folder_name 参数。

        Returns:
            Any: 返回值。
        """
        folders = self.get_mail_folders()
        for folder in folders:
            if folder.get('displayName') == folder_name:
                return folder.get('id')
        return None

    def get_mail_detail(self, mail_id: str):
        """
        获取mail detail。

        Args:
            mail_id: 资源 ID。

        Returns:
            Any: 返回值。
        """
        params = {
            "$select": "body,toRecipients,internetMessageId,internetMessageHeaders"
        }
        response = self.session.get(
            url=f"{self.base_url}/users/{self.email_address}/messages/{mail_id}",
            headers=self.headers,
            params=params,
            timeout=3
        )
        return response.json().get('message')

    def get_mail_list(self, folder_name: str='inbox', size: int=100):
        """
        获取mail list。

        Args:
            folder_name: folder_name 参数。
            size: size 参数。

        Returns:
            Any: 返回值。
        """
        folder_id = self.get_folder_id(folder_name=folder_name)
        params = {
            "size": size,
            # "$select": "toRecipients"
        }
        response = self.session.get(
            url=f"{self.base_url}/users/{self.email_address}/mailFolders/{folder_id}/messages",
            headers=self.headers,
            params=params,
            timeout=3
        )
        messages = response.json().get("messages")
        sent_to_list = []
        for message in messages:
            mail_id = message.get("id")
            detail = self.get_mail_detail(mail_id=mail_id)
            
            for to_recipient in detail.get('toRecipients'):
                sent_to_list.append(to_recipient.get('email'))
            
            yield MailDetail(
                uid=message.get('id'),
                sent_from=message.get('from').get('email'),
                sent_to=sent_to_list,
                date=TimeUtils.convert_str_to_datetime(time_str=message.get('sentDateTime'), app='alimail'),
                cc="",
                subject=message.get('subject'),
                body_plain=message.get('summary'),
                body_html=""
            )

    def move(self, uid: str, destination_folder: str) -> ReturnResponse:
        """
        执行 move 相关逻辑。

        Args:
            uid: uid 参数。
            destination_folder: destination_folder 参数。

        Returns:
            Any: 返回值。
        """
        params = {
            "ids": [uid],
            "destinationFolderId": self.get_folder_id(destination_folder)
        }
        r = self.session.post(
            url=f"{self.base_url}/users/{self.email_address}/messages/move",
            params=params
        )
        if r.status_code == 200:
            return ReturnResponse(code=0, msg=f'邮件移动到 {destination_folder} 成功', data=None)
        else:
            return ReturnResponse(code=1, msg=f'邮件移动到 {destination_folder} 失败', data=r.json())

if __name__ == '__main__':
    ali_mail = AliMail()



#!/usr/bin/env python3

import json
from pytbox.utils.response import ReturnResponse


def load_dev_file(file_path: str) -> ReturnResponse:
    """从开发环境文件加载数据并返回 ReturnResponse 对象
    
    Args:
        file_path: JSON 文件路径
        
    Returns:
        ReturnResponse: 包装后的响应对象
    """
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            data = json.load(f)
        
        # 如果已经是 ReturnResponse 格式的字典，直接转换
        if isinstance(data, dict) and 'code' in data and 'msg' in data:
            return ReturnResponse(
                code=data.get('code', 0),
                msg=data.get('msg', ''),
                data=data.get('data', None)
            )
        else:
            # 如果是其他格式，包装成成功响应
            return ReturnResponse(
                code=0,
                msg='开发文件加载成功',
                data=data
            )
    except FileNotFoundError:
        return ReturnResponse(
            code=4,
            msg=f'开发文件未找到: {file_path}',
            data=None
        )
    except json.JSONDecodeError as e:
        return ReturnResponse(
            code=1,
            msg=f'JSON 解析错误: {str(e)}',
            data=None
        )


"""Common contracts and helpers for collector-style cloud REST data.

This module intentionally keeps a lightweight runtime surface:
- No external IO
- No framework-specific dependency
- TypedDict/dataclass contracts plus pure helper functions
"""

from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime, timezone
from math import ceil
from typing import Any, Generic, Iterable, Mapping, Sequence, TypeVar, TypedDict


T = TypeVar("T")


class InstanceContract(TypedDict, total=False):
    """Minimal instance contract shared by ECS/RDS/Redis."""

    id: str
    name: str
    status: str
    region: str
    created_at: str
    expired_at: str | None
    tags: dict[str, str]


class OrphanDiskContract(TypedDict, total=False):
    """Minimal orphan disk contract."""

    disk_id: str
    size_gb: int
    region: str
    created_at: str
    last_attached_at: str | None


class OrphanEipContract(TypedDict, total=False):
    """Minimal orphan EIP contract."""

    eip: str
    region: str
    bandwidth: int | None
    created_at: str


class StatsContract(TypedDict, total=False):
    """Common stats contract."""

    total: int
    by_status: dict[str, int]
    by_region: dict[str, int]


class StoragePointContract(TypedDict):
    """Timeseries point for storage metrics."""

    ts: str
    used_gb: float
    total_gb: float
    usage_pct: float


class AlertsStatsContract(TypedDict, total=False):
    """Security center alerts stats contract."""

    total: int
    by_severity: dict[str, int]
    by_type: dict[str, int]


class AccessKeyAuditContract(TypedDict, total=False):
    """Identity access key audit contract."""

    id: str
    created_at: str
    last_used_at: str | None


class UserAuditContract(TypedDict, total=False):
    """Identity user audit contract."""

    user_id: str
    user_name: str
    last_login_at: str | None
    mfa_enabled: bool
    access_keys: list[AccessKeyAuditContract]


@dataclass(frozen=True)
class PageResult(Generic[T]):
    """Pagination result for deterministic page-based list APIs.

    Attributes:
        items: Current page items.
        page: 1-based current page.
        page_size: Page size used.
        total: Total item count before slicing.
        total_pages: Total pages (0 if total == 0).
    """

    items: list[T]
    page: int
    page_size: int
    total: int
    total_pages: int


def iso8601_now() -> str:
    """Return current UTC time in ISO8601 with trailing 'Z'."""

    return datetime.now(timezone.utc).replace(microsecond=0).isoformat().replace("+00:00", "Z")


def paginate_items(items: Sequence[T], page: int, page_size: int) -> PageResult[T]:
    """Paginate a sequence deterministically.

    Args:
        items: Source sequence.
        page: 1-based page number.
        page_size: Positive page size.

    Returns:
        A PageResult carrying sliced items and page metadata.

    Raises:
        ValueError: If page < 1 or page_size < 1.
    """

    if page < 1:
        raise ValueError("page must be >= 1")
    if page_size < 1:
        raise ValueError("page_size must be >= 1")

    total = len(items)
    total_pages = ceil(total / page_size) if total else 0
    start = (page - 1) * page_size
    end = start + page_size
    page_items = list(items[start:end]) if start < total else []
    return PageResult(items=page_items, page=page, page_size=page_size, total=total, total_pages=total_pages)


def count_by(values: Iterable[str | None]) -> dict[str, int]:
    """Count string values.

    Empty strings or None values are skipped.
    """

    output: dict[str, int] = {}
    for raw in values:
        key = (raw or "").strip()
        if not key:
            continue
        output[key] = output.get(key, 0) + 1
    return output


def build_stats(
    items: Sequence[Mapping[str, Any]],
    *,
    status_key: str = "status",
    region_key: str = "region",
) -> StatsContract:
    """Build standard stats structure from mapping-like items.

    Args:
        items: Resource items.
        status_key: Item key for status.
        region_key: Item key for region.

    Returns:
        Stats contract with total/by_status/by_region.
    """

    by_status = count_by(str(item.get(status_key, "")).strip() for item in items)
    by_region = count_by(str(item.get(region_key, "")).strip() for item in items)
    return StatsContract(total=len(items), by_status=by_status, by_region=by_region)


__all__ = [
    "AccessKeyAuditContract",
    "AlertsStatsContract",
    "InstanceContract",
    "OrphanDiskContract",
    "OrphanEipContract",
    "PageResult",
    "StatsContract",
    "StoragePointContract",
    "UserAuditContract",
    "build_stats",
    "count_by",
    "iso8601_now",
    "paginate_items",
]



#!/usr/bin/env python3
"""1Password Connect client wrapper with retry and idempotency safeguards."""

from __future__ import annotations

import hashlib
import json
import logging
import time
from typing import Any, Callable, Dict, List, Optional, Tuple, TypeVar

from onepasswordconnectsdk.client import new_client_from_environment
from onepasswordconnectsdk.models import Field, Item

T = TypeVar("T")


class OnePasswordConnect:
    """Sync wrapper for common 1Password Connect item operations."""

    def __init__(
        self,
        vault_id: str,
        request_timeout: float = 5.0,
        max_retries: int = 3,
        retry_backoff_base: float = 0.5,
        idempotency_ttl_seconds: int = 300,
    ) -> None:
        """Initialize client wrapper.

        Args:
            vault_id: Target vault ID for all operations.
            request_timeout: HTTP request timeout in seconds.
            max_retries: Max retry attempts. Value is capped to 3.
            retry_backoff_base: Base seconds for exponential backoff.
            idempotency_ttl_seconds: In-memory idempotency TTL for writes.
        """
        self.client = new_client_from_environment()
        self.vault_id = vault_id
        self.request_timeout = request_timeout
        self.max_retries = max(1, min(max_retries, 3))
        self.retry_backoff_base = retry_backoff_base
        self.idempotency_ttl_seconds = idempotency_ttl_seconds
        self._idempotency_cache: Dict[str, Tuple[float, Any]] = {}
        self.logger = logging.getLogger(__name__)
        self._configure_timeout()

    def create_item(
        self,
        name: str,
        username: Optional[str] = None,
        password: Optional[str] = None,
        notes: str = "create by automation",
        tags: Optional[List[str]] = None,
    ) -> Any:
        """Create a login item in the configured vault.

        Args:
            name: Item title.
            username: Username value.
            password: Password value.
            notes: Note value.
            tags: Optional tag list.

        Returns:
            Any: Created 1Password item object.
        """
        key = self._build_idempotency_key(
            operation="create_item",
            parts=[name, username, password, notes, tags],
        )

        def _caller() -> Any:
            new_item = Item(
                title=name,
                category="LOGIN",
                tags=tags,
                fields=[
                    Field(value=username, purpose="USERNAME"),
                    Field(value=password, purpose="PASSWORD"),
                    Field(value=notes, purpose="NOTES"),
                ],
            )
            return self._execute_with_retry(
                action=lambda: self.client.create_item(self.vault_id, new_item),
                task_id=name or "-",
                target="create_item",
            )

        return self._run_idempotent(key=key, caller=_caller)

    def delete_item(self, item_id: str) -> Any:
        """Delete item by ID.

        Args:
            item_id: Item ID.

        Returns:
            Any: SDK delete response, usually ``None``.
        """
        key = self._build_idempotency_key(operation="delete_item", parts=[item_id])
        return self._run_idempotent(
            key=key,
            caller=lambda: self._execute_with_retry(
                action=lambda: self.client.delete_item(item_id, self.vault_id),
                task_id=item_id,
                target="delete_item",
            ),
        )

    def get_item(self, item_id: str) -> Any:
        """Get item detail by ID.

        Args:
            item_id: Item ID.

        Returns:
            Any: 1Password item object.
        """
        return self._execute_with_retry(
            action=lambda: self.client.get_item(item_id, self.vault_id),
            task_id=item_id,
            target="get_item",
        )

    def get_item_by_title(self, title: str = "", totp: bool = False) -> Any:
        """Get item fields by title or return the first TOTP value.

        Args:
            title: Item title.
            totp: Whether to return TOTP value directly.

        Returns:
            Any: Dict of fields when ``totp=False``; TOTP string or ``None`` when ``totp=True``.
        """
        item = self._execute_with_retry(
            action=lambda: self.client.get_item_by_title(title, self.vault_id),
            task_id=title or "-",
            target="get_item_by_title",
        )

        if totp:
            for field in item.fields:
                if getattr(field, "totp", None) is not None:
                    return field.totp
            return None

        value: Dict[str, Any] = {}
        for field in item.fields:
            label = getattr(field, "label", None) or getattr(field, "purpose", None)
            if label is not None:
                value[label] = getattr(field, "value", None)
        return value

    def update_item(
        self,
        item_id: str,
        name: Optional[str] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        tags: Optional[List[str]] = None,
        notes: Optional[str] = None,
    ) -> Any:
        """Update an existing item by ID.

        Args:
            item_id: Item ID.
            name: New title when provided.
            username: New username when provided.
            password: New password when provided.
            tags: New tag list. ``[]`` is allowed and means clear tags.
            notes: New notes when provided.

        Returns:
            Any: Updated 1Password item object.
        """
        key = self._build_idempotency_key(
            operation="update_item",
            parts=[item_id, name, username, password, tags, notes],
        )

        def _caller() -> Any:
            update_target = self.get_item(item_id=item_id)

            if name is not None:
                update_target.title = name

            for field in update_target.fields:
                purpose = getattr(field, "purpose", None)
                if purpose == "USERNAME" and username is not None:
                    field.value = username
                if purpose == "PASSWORD" and password is not None:
                    field.value = password
                if purpose == "NOTES" and notes is not None:
                    field.value = notes

            if tags is not None:
                update_target.tags = tags

            return self._execute_with_retry(
                action=lambda: self.client.update_item(item_id, self.vault_id, update_target),
                task_id=item_id,
                target="update_item",
            )

        return self._run_idempotent(key=key, caller=_caller)

    def search_item(self, title: Optional[str] = None, tag: Optional[str] = None) -> list[Any]:
        """Search items by title, tag, or both.

        Args:
            title: Title filter.
            tag: Tag filter.

        Returns:
            list[Any]: Matched item summaries.
        """
        filter_query = self._build_filter_query(title=title, tag=tag)
        return self._execute_with_retry(
            action=lambda: self.client.get_items(self.vault_id, filter_query=filter_query),
            task_id=title or tag or "-",
            target="search_item",
        )

    def _configure_timeout(self) -> None:
        """Apply explicit timeout on SDK HTTP session when available."""
        session = getattr(self.client, "session", None)
        if session is None:
            return
        try:
            session.timeout = self.request_timeout
        except Exception as exc:  # pragma: no cover - defensive; depends on SDK internals.
            self.logger.debug("failed to set timeout: %s", exc.__class__.__name__)

    def _execute_with_retry(
        self,
        action: Callable[[], T],
        task_id: str,
        target: str,
    ) -> T:
        """Execute an external request with bounded retries.

        Args:
            action: Callable that performs the SDK request.
            task_id: Task identifier for operation logs.
            target: Target operation name.

        Returns:
            T: Value returned by ``action``.

        Raises:
            Exception: Re-raises the final request exception after retries.
        """
        for attempt in range(1, self.max_retries + 1):
            started_at = time.monotonic()
            try:
                result = action()
                duration_ms = int((time.monotonic() - started_at) * 1000)
                self._log_step(
                    task_id=task_id,
                    target=target,
                    result=f"success_attempt_{attempt}",
                    duration_ms=duration_ms,
                )
                return result
            except Exception as exc:
                duration_ms = int((time.monotonic() - started_at) * 1000)
                result = f"error_{exc.__class__.__name__}_attempt_{attempt}"
                self._log_step(
                    task_id=task_id,
                    target=target,
                    result=result,
                    duration_ms=duration_ms,
                )
                if attempt >= self.max_retries:
                    raise
                time.sleep(self.retry_backoff_base * (2 ** (attempt - 1)))
        raise RuntimeError("unexpected retry loop exit")

    def _build_filter_query(self, title: Optional[str], tag: Optional[str]) -> Optional[str]:
        """Build item query with optional title/tag filters.

        Args:
            title: Title filter value.
            tag: Tag filter value.

        Returns:
            Optional[str]: SDK filter query string, or ``None`` when no filters are set.
        """
        clauses: List[str] = []
        if title:
            clauses.append(f'title eq "{self._escape_filter_value(title)}"')
        if tag:
            clauses.append(f'tag eq "{self._escape_filter_value(tag)}"')
        if not clauses:
            return None
        return " and ".join(clauses)

    def _escape_filter_value(self, value: str) -> str:
        """Escape filter value for double-quoted query string.

        Args:
            value: Raw filter value.

        Returns:
            str: Escaped value.
        """
        return value.replace('"', '\\"')

    def _build_idempotency_key(self, operation: str, parts: List[Any]) -> Optional[str]:
        """Build idempotency cache key for write operations.

        Args:
            operation: Operation name.
            parts: Value list that identifies a logical write request.

        Returns:
            Optional[str]: Hash key in current TTL window, or ``None`` when disabled.
        """
        if self.idempotency_ttl_seconds <= 0:
            return None
        window = int(time.time() // self.idempotency_ttl_seconds)
        payload = [operation, window, parts]
        serialized = json.dumps(payload, sort_keys=True, ensure_ascii=True, default=str)
        return hashlib.sha256(serialized.encode("utf-8")).hexdigest()

    def _run_idempotent(self, key: Optional[str], caller: Callable[[], T]) -> T:
        """Execute write operation with in-memory idempotency cache.

        Args:
            key: Idempotency key. ``None`` disables cache.
            caller: Callable that performs the write request.

        Returns:
            T: Cached or fresh result.
        """
        if key is None:
            return caller()

        now = time.time()
        self._cleanup_idempotency_cache(now_ts=now)
        cached = self._idempotency_cache.get(key)
        if cached and now - cached[0] <= self.idempotency_ttl_seconds:
            return cached[1]

        result = caller()
        self._idempotency_cache[key] = (now, result)
        return result

    def _cleanup_idempotency_cache(self, now_ts: float) -> None:
        """Remove expired idempotency cache records.

        Args:
            now_ts: Current UNIX timestamp.
        """
        if self.idempotency_ttl_seconds <= 0:
            self._idempotency_cache.clear()
            return
        expired_keys = [
            cache_key
            for cache_key, (created_at, _resp) in self._idempotency_cache.items()
            if now_ts - created_at > self.idempotency_ttl_seconds
        ]
        for cache_key in expired_keys:
            self._idempotency_cache.pop(cache_key, None)

    def _log_step(
        self,
        task_id: str,
        target: str,
        result: str,
        duration_ms: int,
    ) -> None:
        """Write key-step operation log.

        Args:
            task_id: Operation task ID.
            target: Target method or endpoint.
            result: Result summary.
            duration_ms: Cost in milliseconds.
        """
        self.logger.info(
            "[onepassword_connect] task_id=%s target=%s result=%s duration_ms=%s",
            task_id,
            target,
            result,
            duration_ms,
        )


if __name__ == "__main__":
    pass



#!/usr/bin/env python3

"""Network backup config loader."""

from __future__ import annotations

import json
import logging
import time
import uuid
from pathlib import Path
from typing import Any, Dict

from pytbox.schemas.codes import RespCode
from pytbox.schemas.response import ReturnResponse

try:
    import tomllib  # type: ignore[attr-defined]
except ModuleNotFoundError:  # pragma: no cover
    tomllib = None  # type: ignore[assignment]

try:
    import tomli  # type: ignore
except ModuleNotFoundError:  # pragma: no cover
    tomli = None  # type: ignore[assignment]

try:
    import toml  # type: ignore
except ModuleNotFoundError:  # pragma: no cover
    toml = None  # type: ignore[assignment]


DEFAULT_OUTPUT_DIR = "./backups/network"
DEFAULT_TIMEOUT = 10
DEFAULT_RETRIES = 3

_LOGGER = logging.getLogger(__name__)


def _log_step(task_id: str, target: str, result: str, start_ts: float) -> None:
    """Log key steps for external IO.

    Args:
        task_id: Correlation id.
        target: Config file target.
        result: Step result.
        start_ts: Monotonic start timestamp.
    """
    duration_ms = int((time.monotonic() - start_ts) * 1000)
    _LOGGER.info(
        "task_id=%s target=%s result=%s duration_ms=%s",
        task_id,
        target,
        result,
        duration_ms,
    )


def _normalize_retries(value: Any) -> int:
    """Normalize retries into [1, 3].

    Args:
        value: Raw retries value.

    Returns:
        int: Normalized retries.
    """
    try:
        retries = int(value)
    except (TypeError, ValueError):
        retries = DEFAULT_RETRIES
    return min(max(retries, 1), 3)


def _normalize_timeout(value: Any) -> int:
    """Normalize timeout into a positive integer.

    Args:
        value: Raw timeout value.

    Returns:
        int: Normalized timeout.
    """
    try:
        timeout = int(value)
    except (TypeError, ValueError):
        timeout = DEFAULT_TIMEOUT
    return timeout if timeout > 0 else DEFAULT_TIMEOUT


def _validate_device(device: Dict[str, Any], index: int) -> ReturnResponse:
    """Validate one device config item.

    Args:
        device: Device config item.
        index: Device index in config.

    Returns:
        ReturnResponse: Validation result.
    """
    required_fields = ("ip", "os", "protocol", "username", "password")
    missing = [field for field in required_fields if not device.get(field)]
    if missing:
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg=f"devices[{index}] 缺少字段: {', '.join(missing)}",
        )

    protocol = str(device.get("protocol", "")).lower()
    if protocol not in {"ssh", "telnet"}:
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg=f"devices[{index}] protocol 仅支持 ssh/telnet",
        )

    return ReturnResponse.ok()


def _normalize_device(device: Dict[str, Any]) -> Dict[str, Any]:
    """Normalize one device config item.

    Args:
        device: Raw device config item.

    Returns:
        dict[str, Any]: Normalized device config.
    """
    protocol = str(device.get("protocol", "ssh")).lower()
    default_port = 22 if protocol == "ssh" else 23
    normalized = {
        "ip": str(device["ip"]).strip(),
        "os": str(device["os"]).strip().lower(),
        "protocol": protocol,
        "username": str(device["username"]),
        "password": str(device["password"]),
        "port": int(device.get("port", default_port)),
    }
    if device.get("enable_password"):
        normalized["enable_password"] = str(device["enable_password"])
    if device.get("backup_command"):
        normalized["backup_command"] = str(device["backup_command"])
    if device.get("disable_paging_command"):
        normalized["disable_paging_command"] = str(device["disable_paging_command"])
    if device.get("device_type"):
        normalized["device_type"] = str(device["device_type"])
    return normalized


def _load_yaml(path: Path) -> ReturnResponse:
    """Load YAML config.

    Args:
        path: Config file path.

    Returns:
        ReturnResponse: Parsed config.
    """
    try:
        import yaml  # type: ignore
    except ModuleNotFoundError:
        return ReturnResponse.fail(
            code=RespCode.INTERNAL_ERROR,
            msg="未安装 pyyaml，无法解析 YAML 配置",
        )

    try:
        with path.open("r", encoding="utf-8") as handle:
            data = yaml.safe_load(handle) or {}
        if not isinstance(data, dict):
            return ReturnResponse.fail(
                code=RespCode.INVALID_PARAMS,
                msg="配置文件内容必须是对象结构",
            )
        return ReturnResponse.ok(data=data)
    except OSError as exc:
        return ReturnResponse.fail(
            code=RespCode.INTERNAL_ERROR,
            msg=f"读取配置文件失败: {exc}",
        )


def _load_json(path: Path) -> ReturnResponse:
    """Load JSON config.

    Args:
        path: Config file path.

    Returns:
        ReturnResponse: Parsed config.
    """
    try:
        with path.open("r", encoding="utf-8") as handle:
            data = json.load(handle)
        if not isinstance(data, dict):
            return ReturnResponse.fail(
                code=RespCode.INVALID_PARAMS,
                msg="配置文件内容必须是对象结构",
            )
        return ReturnResponse.ok(data=data)
    except (OSError, json.JSONDecodeError) as exc:
        return ReturnResponse.fail(
            code=RespCode.INTERNAL_ERROR,
            msg=f"读取配置文件失败: {exc}",
        )


def _load_toml(path: Path) -> ReturnResponse:
    """Load TOML config.

    Args:
        path: Config file path.

    Returns:
        ReturnResponse: Parsed config.
    """
    try:
        if tomllib is not None:
            with path.open("rb") as handle:
                data = tomllib.load(handle)
        elif tomli is not None:  # pragma: no cover
            with path.open("rb") as handle:
                data = tomli.load(handle)
        elif toml is not None:  # pragma: no cover
            with path.open("r", encoding="utf-8") as handle:
                data = toml.load(handle)
        else:
            return ReturnResponse.fail(
                code=RespCode.INTERNAL_ERROR,
                msg="未安装 TOML 解析依赖",
            )
    except OSError as exc:
        return ReturnResponse.fail(
            code=RespCode.INTERNAL_ERROR,
            msg=f"读取配置文件失败: {exc}",
        )
    except Exception as exc:  # noqa: BLE001
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg=f"TOML 配置解析失败: {exc}",
        )

    if not isinstance(data, dict):
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg="配置文件内容必须是对象结构",
        )
    return ReturnResponse.ok(data=data)


def load_backup_config(path: str) -> ReturnResponse:
    """Load and validate backup config from file.

    Args:
        path: Config file path. Supports yaml/yml/json/toml.

    Returns:
        ReturnResponse: Loaded and normalized config.
    """
    config_path = Path(path).expanduser()
    if not config_path.exists():
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg=f"配置文件不存在: {config_path}",
        )

    task_id = uuid.uuid4().hex[:8]
    start_ts = time.monotonic()
    suffix = config_path.suffix.lower()

    if suffix in {".yaml", ".yml"}:
        loaded = _load_yaml(config_path)
    elif suffix == ".json":
        loaded = _load_json(config_path)
    elif suffix == ".toml":
        loaded = _load_toml(config_path)
    else:
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg=f"不支持的配置格式: {suffix}",
        )

    if loaded.code != 0:
        _log_step(task_id, str(config_path), "fail", start_ts)
        return loaded

    raw = loaded.data if isinstance(loaded.data, dict) else {}
    devices = raw.get("devices", [])
    if devices is None:
        devices = []
    if not isinstance(devices, list):
        _log_step(task_id, str(config_path), "fail", start_ts)
        return ReturnResponse.fail(
            code=RespCode.INVALID_PARAMS,
            msg="devices 必须是列表",
        )

    normalized_devices = []
    for index, item in enumerate(devices):
        if not isinstance(item, dict):
            _log_step(task_id, str(config_path), "fail", start_ts)
            return ReturnResponse.fail(
                code=RespCode.INVALID_PARAMS,
                msg=f"devices[{index}] 必须是对象",
            )
        validation = _validate_device(item, index)
        if validation.code != 0:
            _log_step(task_id, str(config_path), "fail", start_ts)
            return validation
        normalized_devices.append(_normalize_device(item))

    normalized = {
        "output_dir": str(raw.get("output_dir") or DEFAULT_OUTPUT_DIR),
        "timeout": _normalize_timeout(raw.get("timeout")),
        "retries": _normalize_retries(raw.get("retries")),
        "devices": normalized_devices,
    }
    _log_step(task_id, str(config_path), "ok", start_ts)
    return ReturnResponse.ok(data=normalized, msg="配置加载成功")

